from . import AWSObject, AWSProperty
from .validators import integer_list_item
from .constants import LOGS_ALLOWED_RETENTION_DAYS as RETENTION_DAYS


class Destination(AWSObject):
    resource_type = "AWS::Logs::Destination"

    props = {
        'DestinationName': (basestring, True),
        'DestinationPolicy': (basestring, True),
        'RoleArn': (basestring, True),
        'TargetArn': (basestring, True),
    }


class LogGroup(AWSObject):
    resource_type = "AWS::Logs::LogGroup"

    props = {
        'LogGroupName': (basestring, False),
        'RetentionInDays': (integer_list_item(RETENTION_DAYS), False),
    }


class LogStream(AWSObject):
    resource_type = "AWS::Logs::LogStream"

    props = {
        'LogGroupName': (basestring, True),
        'LogStreamName': (basestring, False)
    }


class MetricTransformation(AWSProperty):
    props = {
        'DefaultValue': (float, False),
        'MetricName': (basestring, True),
        'MetricNamespace': (basestring, True),
        'MetricValue': (basestring, True),
    }


class MetricFilter(AWSObject):
    resource_type = "AWS::Logs::MetricFilter"

    props = {
        'FilterPattern': (basestring, True),
        'LogGroupName': (basestring, True),
        'MetricTransformations': ([MetricTransformation], True),
    }


class SubscriptionFilter(AWSObject):
    resource_type = "AWS::Logs::SubscriptionFilter"

    props = {
        'DestinationArn': (basestring, True),
        'FilterPattern': (basestring, True),
        'LogGroupName': (basestring, True),
        'RoleArn': (basestring, False),
    }



# Copyright (c) 2012-2017, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty
from .validators import boolean, integer


class ApiKey(AWSObject):
    resource_type = "AWS::AppSync::ApiKey"

    props = {
        'ApiId': (basestring, True),
        'Description': (basestring, False),
        'Expires': (integer, False),
    }


class DynamoDBConfig(AWSProperty):
    props = {
        'AwsRegion': (basestring, True),
        'TableName': (basestring, True),
        'UseCallerCredentials': (boolean, False),
    }


class ElasticsearchConfig(AWSProperty):
    props = {
        'AwsRegion': (basestring, True),
        'Endpoint': (basestring, True),
    }


class LambdaConfig(AWSProperty):
    props = {
        'LambdaFunctionArn': (basestring, True),
    }


class DataSource(AWSObject):
    resource_type = "AWS::AppSync::DataSource"

    props = {
        'ApiId': (basestring, True),
        'Description': (basestring, False),
        'DynamoDBConfig': (DynamoDBConfig, False),
        'ElasticsearchConfig': (ElasticsearchConfig, False),
        'LambdaConfig': (LambdaConfig, False),
        'Name': (basestring, True),
        'ServiceRoleArn': (basestring, False),
        'Type': (basestring, True),
    }


class LogConfig(AWSProperty):
    props = {
        'CloudWatchLogsRoleArn': (basestring, False),
        'FieldLogLevel': (basestring, False),
    }


class OpenIDConnectConfig(AWSProperty):
    props = {
        'AuthTTL': (float, False),
        'ClientId': (basestring, False),
        'IatTTL': (float, False),
        'Issuer': (basestring, True),
    }


class UserPoolConfig(AWSProperty):
    props = {
        'AppIdClientRegex': (basestring, False),
        'AwsRegion': (basestring, False),
        'DefaultAction': (basestring, False),
        'UserPoolId': (basestring, False),
    }


class GraphQLApi(AWSObject):
    resource_type = "AWS::AppSync::GraphQLApi"

    props = {
        'AuthenticationType': (basestring, True),
        'LogConfig': (LogConfig, False),
        'Name': (basestring, True),
        'OpenIDConnectConfig': (OpenIDConnectConfig, False),
        'UserPoolConfig': (UserPoolConfig, False),
    }


class GraphQLSchema(AWSObject):
    resource_type = "AWS::AppSync::GraphQLSchema"

    props = {
        'ApiId': (basestring, True),
        'Definition': (basestring, False),
        'DefinitionS3Location': (basestring, False),
    }


class Resolver(AWSObject):
    resource_type = "AWS::AppSync::Resolver"

    props = {
        'ApiId': (basestring, True),
        'DataSourceName': (basestring, True),
        'FieldName': (basestring, True),
        'RequestMappingTemplate': (basestring, False),
        'RequestMappingTemplateS3Location': (basestring, False),
        'ResponseMappingTemplate': (basestring, False),
        'ResponseMappingTemplateS3Location': (basestring, False),
        'TypeName': (basestring, True),
    }



# Converted from ElasticBeanstalk_Nodejs.template located at:
# http://aws.amazon.com/cloudformation/aws-cloudformation-templates/

from troposphere import (
    GetAtt, Join, Output,
    Parameter, Ref, Template, FindInMap
)

from troposphere.elasticbeanstalk import (
    Application, ApplicationVersion, ConfigurationTemplate, Environment,
    SourceBundle, OptionSettings
)

from troposphere.iam import Role, InstanceProfile
from troposphere.iam import PolicyType as IAMPolicy

from awacs.aws import Allow, Statement, Action, Principal, Policy
from awacs.sts import AssumeRole


t = Template()

t.add_version()

t.add_description(
    "AWS CloudFormation Sample Template ElasticBeanstalk_Nodejs_Sample: "
    "Configure and launch the AWS Elastic Beanstalk sample application. "
    "**WARNING** This template creates one or more Amazon EC2 instances. You "
    "will be billed for the AWS resources used if you create a stack from "
    "this template.")

keyname = t.add_parameter(Parameter(
    "KeyName",
    Description="Name of an existing EC2 KeyPair to enable SSH access to "
                "the AWS Elastic Beanstalk instance",
    Type="AWS::EC2::KeyPair::KeyName",
    ConstraintDescription="must be the name of an existing EC2 KeyPair."
))

t.add_mapping("Region2Principal", {
    'ap-northeast-1': {
        'EC2Principal': 'ec2.amazonaws.com',
        'OpsWorksPrincipal': 'opsworks.amazonaws.com'},
    'ap-southeast-1': {
        'EC2Principal': 'ec2.amazonaws.com',
        'OpsWorksPrincipal': 'opsworks.amazonaws.com'},
    'ap-southeast-2': {
        'EC2Principal': 'ec2.amazonaws.com',
        'OpsWorksPrincipal': 'opsworks.amazonaws.com'},
    'cn-north-1': {
        'EC2Principal': 'ec2.amazonaws.com.cn',
        'OpsWorksPrincipal': 'opsworks.amazonaws.com.cn'},
    'eu-central-1': {
        'EC2Principal': 'ec2.amazonaws.com',
        'OpsWorksPrincipal': 'opsworks.amazonaws.com'},
    'eu-west-1': {
        'EC2Principal': 'ec2.amazonaws.com',
        'OpsWorksPrincipal': 'opsworks.amazonaws.com'},
    'sa-east-1': {
        'EC2Principal': 'ec2.amazonaws.com',
        'OpsWorksPrincipal': 'opsworks.amazonaws.com'},
    'us-east-1': {
        'EC2Principal': 'ec2.amazonaws.com',
        'OpsWorksPrincipal': 'opsworks.amazonaws.com'},
    'us-west-1': {
        'EC2Principal': 'ec2.amazonaws.com',
        'OpsWorksPrincipal': 'opsworks.amazonaws.com'},
    'us-west-2': {
        'EC2Principal': 'ec2.amazonaws.com',
        'OpsWorksPrincipal': 'opsworks.amazonaws.com'}
    }
)

t.add_resource(Role(
    "WebServerRole",
    AssumeRolePolicyDocument=Policy(
        Statement=[
            Statement(
                Effect=Allow, Action=[AssumeRole],
                Principal=Principal(
                    "Service", [
                        FindInMap(
                            "Region2Principal",
                            Ref("AWS::Region"), "EC2Principal")
                    ]
                )
            )
        ]
    ),
    Path="/"
))

t.add_resource(IAMPolicy(
    "WebServerRolePolicy",
    PolicyName="WebServerRole",
    PolicyDocument=Policy(
        Statement=[
            Statement(Effect=Allow, NotAction=Action("iam", "*"),
                      Resource=["*"])
        ]
    ),
    Roles=[Ref("WebServerRole")]
))

t.add_resource(InstanceProfile(
    "WebServerInstanceProfile",
    Path="/",
    Roles=[Ref("WebServerRole")]
))

t.add_resource(Application(
    "SampleApplication",
    Description="AWS Elastic Beanstalk Sample Node.js Application"
))

t.add_resource(ApplicationVersion(
    "SampleApplicationVersion",
    Description="Version 1.0",
    ApplicationName=Ref("SampleApplication"),
    SourceBundle=SourceBundle(
        S3Bucket=Join("-", ["elasticbeanstalk-samples", Ref("AWS::Region")]),
        S3Key="nodejs-sample.zip"
    )
))

t.add_resource(ConfigurationTemplate(
    "SampleConfigurationTemplate",
    ApplicationName=Ref("SampleApplication"),
    Description="SSH access to Node.JS Application",
    SolutionStackName="64bit Amazon Linux 2014.03 v1.0.9 running Node.js",
    OptionSettings=[
        OptionSettings(
            Namespace="aws:autoscaling:launchconfiguration",
            OptionName="EC2KeyName",
            Value=Ref("KeyName")
        ),
        OptionSettings(
            Namespace="aws:autoscaling:launchconfiguration",
            OptionName="IamInstanceProfile",
            Value=Ref("WebServerInstanceProfile")
        )
    ]
))

t.add_resource(Environment(
    "SampleEnvironment",
    Description="AWS Elastic Beanstalk Environment running Sample Node.js "
                "Application",
    ApplicationName=Ref("SampleApplication"),
    TemplateName=Ref("SampleConfigurationTemplate"),
    VersionLabel=Ref("SampleApplicationVersion")
))

t.add_output(
    Output(
        "URL",
        Description="URL of the AWS Elastic Beanstalk Environment",
        Value=Join("", ["http://", GetAtt("SampleEnvironment", "EndpointURL")])
    )
)

print(t.to_json())



from troposphere import Template
from troposphere.ecr import Repository
from awacs.aws import Allow, Policy, AWSPrincipal, Statement
import awacs.ecr as ecr
import awacs.iam as iam


t = Template()

t.add_resource(
    Repository(
        'MyRepository',
        RepositoryName='test-repository',
        RepositoryPolicyText=Policy(
            Version='2008-10-17',
            Statement=[
                Statement(
                    Sid='AllowPushPull',
                    Effect=Allow,
                    Principal=AWSPrincipal([
                        iam.ARN(account='123456789012', resource='user/Bob'),
                        iam.ARN(account='123456789012', resource='user/Alice'),
                    ]),
                    Action=[
                        ecr.GetDownloadUrlForLayer,
                        ecr.BatchGetImage,
                        ecr.BatchCheckLayerAvailability,
                        ecr.PutImage,
                        ecr.InitiateLayerUpload,
                        ecr.UploadLayerPart,
                        ecr.CompleteLayerUpload,
                    ],
                ),
            ]
        ),
    )
)

print(t.to_json())



from troposphere import Template, Ref
from troposphere.applicationautoscaling import (
    ScalableTarget,
    StepAdjustment,
    StepScalingPolicyConfiguration,
    ScalingPolicy,
)


t = Template()

scalable_target = ScalableTarget(
    'scalableTarget',
    MaxCapacity=2,
    MinCapacity=1,
    ResourceId='service/ecsStack',
    RoleARN='Access Management (IAM) role',
    ScalableDimension='ecs:service:DesiredCount',
    ServiceNamespace='ecs',
)

scaling_policy = ScalingPolicy(
    'scalingPolicy',
    PolicyName='AStepPolicy',
    PolicyType='StepScaling',
    ScalingTargetId=Ref(scalable_target),
    StepScalingPolicyConfiguration=StepScalingPolicyConfiguration(
        AdjustmentType='PercentChangeInCapacity',
        Cooldown=60,
        MetricAggregationType='Average',
        StepAdjustments=[
            StepAdjustment(
                MetricIntervalLowerBound=0,
                ScalingAdjustment=200,
            ),
        ],
    ),
)

t = Template()

t.add_resource(scalable_target)
t.add_resource(scaling_policy)

print(t.to_json())



from . import AWSObject, AWSProperty
from .validators import boolean
try:
    from awacs.aws import Policy
    policytypes = (dict, Policy)
except ImportError:
    policytypes = dict,


class CloudwatchAlarmAction(AWSProperty):
    props = {
        'AlarmName': (basestring, True),
        'RoleArn': (basestring, True),
        'StateReason': (basestring, True),
        'StateValue': (basestring, True),
    }


class CloudwatchMetricAction(AWSProperty):
    props = {
        'MetricName': (basestring, True),
        'MetricNamespace': (basestring, True),
        'MetricTimestamp': (basestring, False),
        'MetricUnit': (basestring, True),
        'MetricValue': (basestring, True),
        'RoleArn': (basestring, True),
    }


class DynamoDBAction(AWSProperty):
    props = {
        'HashKeyField': (basestring, True),
        'HashKeyType': (basestring, False),
        'HashKeyValue': (basestring, True),
        'PayloadField': (basestring, False),
        'RangeKeyField': (basestring, False),
        'RangeKeyType': (basestring, False),
        'RangeKeyValue': (basestring, False),
        'RoleArn': (basestring, True),
        'TableName': (basestring, True),
    }


class PutItemInput(AWSProperty):
    props = {
        'TableName': (basestring, True),
    }


class DynamoDBv2Action(AWSProperty):
    props = {
        'PutItem': (PutItemInput, False),
        'RoleArn': (basestring, False),
    }


class ElasticsearchAction(AWSProperty):
    props = {
        'Endpoint': (basestring, True),
        'Id': (basestring, True),
        'Index': (basestring, True),
        'RoleArn': (basestring, True),
        'Type': (basestring, True),
    }


class FirehoseAction(AWSProperty):
    props = {
        'DeliveryStreamName': (basestring, True),
        'RoleArn': (basestring, True),
        'Separator': (basestring, False),
    }


class KinesisAction(AWSProperty):
    props = {
        'PartitionKey': (basestring, False),
        'RoleArn': (basestring, True),
        'StreamName': (basestring, True),
    }


class LambdaAction(AWSProperty):
    props = {
        'FunctionArn': (basestring, True),
    }


class RepublishAction(AWSProperty):
    props = {
        'RoleArn': (basestring, True),
        'Topic': (basestring, True),
    }


class S3Action(AWSProperty):
    props = {
        'BucketName': (basestring, True),
        'Key': (basestring, True),
        'RoleArn': (basestring, True),
    }


class SnsAction(AWSProperty):
    props = {
        'MessageFormat': (basestring, False),
        'RoleArn': (basestring, True),
        'TargetArn': (basestring, True),
    }


class SqsAction(AWSProperty):
    props = {
        'QueueUrl': (basestring, True),
        'RoleArn': (basestring, True),
        'UseBase64': (basestring, False),
    }


class Action(AWSProperty):
    props = {
        'CloudwatchAlarm': (CloudwatchAlarmAction, False),
        'CloudwatchMetric': (CloudwatchMetricAction, False),
        'DynamoDB': (DynamoDBAction, False),
        'DynamoDBv2': (DynamoDBv2Action, False),
        'Elasticsearch': (ElasticsearchAction, False),
        'Firehose': (FirehoseAction, False),
        'Kinesis': (KinesisAction, False),
        'Lambda': (LambdaAction, False),
        'Republish': (RepublishAction, False),
        'S3': (S3Action, False),
        'Sns': (SnsAction, False),
        'Sqs': (SqsAction, False),
    }


class TopicRulePayload(AWSProperty):
    props = {
        'Actions': ([Action], True),
        'AwsIotSqlVersion': (basestring, False),
        'Description': (basestring, False),
        'RuleDisabled': (boolean, True),
        'Sql': (basestring, True),
    }


class TopicRule(AWSObject):
    resource_type = "AWS::IoT::TopicRule"

    props = {
        'RuleName': (basestring, False),
        'TopicRulePayload': (TopicRulePayload, True),
    }


class ThingPrincipalAttachment(AWSObject):
    resource_type = "AWS::IoT::ThingPrincipalAttachment"

    props = {
        'Principal': (basestring, True),
        'ThingName': (basestring, True),
    }


class Thing(AWSObject):
    resource_type = "AWS::IoT::Thing"

    props = {
        'AttributePayload': (dict, False),
        'ThingName': (basestring, False),
    }


class PolicyPrincipalAttachment(AWSObject):
    resource_type = "AWS::IoT::PolicyPrincipalAttachment"

    props = {
        'PolicyName': (basestring, True),
        'Principal': (basestring, True),
    }


class Policy(AWSObject):
    resource_type = "AWS::IoT::Policy"

    props = {
        'PolicyDocument': (policytypes, True),
        'PolicyName': (basestring, False),
    }


class Certificate(AWSObject):
    resource_type = "AWS::IoT::Certificate"

    props = {
        'CertificateSigningRequest': (basestring, True),
        'Status': (basestring, True),
    }



from troposphere import Template
from troposphere.codedeploy import AutoRollbackConfiguration, \
    DeploymentStyle, DeploymentGroup, ElbInfoList, LoadBalancerInfo


template = Template()
template.add_version('2010-09-09')


auto_rollback_configuration = AutoRollbackConfiguration(
    Enabled=True,
    Events=['DEPLOYMENT_FAILURE']
)

deployment_style = DeploymentStyle(
    DeploymentOption='WITH_TRAFFIC_CONTROL'
)

elb_info_list = ElbInfoList(
    Name='DemoLoadBalancer'
)

load_balancer_info = LoadBalancerInfo(
    ElbInfoList=[elb_info_list]
)

deployment_group = DeploymentGroup(
    "DemoDeploymentGroup",
    ApplicationName='DemoApplication',
    AutoRollbackConfiguration=auto_rollback_configuration,
    DeploymentStyle=deployment_style,
    LoadBalancerInfo=load_balancer_info,
    ServiceRoleArn='arn:aws:iam::0123456789:role/codedeploy-role'
)
template.add_resource(deployment_group)

print(template.to_json())



from troposphere import Template
from troposphere.certificatemanager import Certificate, DomainValidationOption

t = Template()

t.add_resource(
    Certificate(
        'mycert',
        DomainName='example.com',
        DomainValidationOptions=[
            DomainValidationOption(
                DomainName='example.com',
                ValidationDomain='example.com',
            ),
        ],
        Tags=[
            {
                'Key': 'tag-key',
                'Value': 'tag-value'
            },
        ],
    )
)

print(t.to_json())



# Converted from S3_Bucket.template located at:
# http://aws.amazon.com/cloudformation/aws-cloudformation-templates/

from troposphere import Output, Ref, Template
from troposphere.s3 import Bucket, PublicRead


t = Template()

t.add_description(
    "AWS CloudFormation Sample Template S3_Bucket: Sample template showing "
    "how to create a publicly accessible S3 bucket. "
    "**WARNING** This template creates an Amazon S3 Bucket. "
    "You will be billed for the AWS resources used if you create "
    "a stack from this template.")

s3bucket = t.add_resource(Bucket("S3Bucket", AccessControl=PublicRead,))

t.add_output(Output(
    "BucketName",
    Value=Ref(s3bucket),
    Description="Name of S3 bucket to hold website content"
))

print(t.to_json())



from troposphere import Parameter, Ref, Template, Tags, If, Equals, Not, Join
from troposphere.constants import KEY_PAIR_NAME, SUBNET_ID, M4_LARGE, NUMBER
import troposphere.emr as emr
import troposphere.iam as iam


scaling_policy = emr.SimpleScalingPolicyConfiguration(
                    AdjustmentType="EXACT_CAPACITY",
                    ScalingAdjustment="1",
                    CoolDown="300"
                )


kms_key = 'arn:aws:kms:us-east-1:123456789012:key/1234-1234-1234-1234-1234'

security_configuration = {
    'EncryptionConfiguration': {
        'EnableInTransitEncryption': 'true',
        'InTransitEncryptionConfiguration': {
            'TLSCertificateConfiguration': {
                'CertificateProviderType': 'PEM',
                'S3Object': 's3://MyConfigStore/artifacts/MyCerts.zip'
            }
        },
        'EnableAtRestEncryption': 'true',
        'AtRestEncryptionConfiguration': {
            'S3EncryptionConfiguration': {
                'EncryptionMode': 'SSE-KMS',
                'AwsKmsKey': kms_key
            },
            'LocalDiskEncryptionConfiguration': {
                'EncryptionKeyProviderType': 'AwsKms',
                'AwsKmsKey': kms_key
            }
        }
    }
}


def generate_rules(rules_name):
    global emr, scaling_policy

    rules = [
        emr.ScalingRule(
            Name=rules_name,
            Description="%s rules" % rules_name,
            Action=emr.ScalingAction(
                Market="ON_DEMAND",
                SimpleScalingPolicyConfiguration=scaling_policy
            ),
            Trigger=emr.ScalingTrigger(
                CloudWatchAlarmDefinition=emr.CloudWatchAlarmDefinition(
                    ComparisonOperator="GREATER_THAN",
                    EvaluationPeriods="120",
                    MetricName="TestMetric",
                    Namespace="AWS/ElasticMapReduce",
                    Period="300",
                    Statistic="AVERAGE",
                    Threshold="50",
                    Unit="PERCENT",
                    Dimensions=[
                        emr.MetricDimension(
                            'my.custom.master.property',
                            'my.custom.master.value'
                        )
                    ]
                )
            )
        )
    ]
    return rules


template = Template()
template.add_description(
    "Sample CloudFormation template for creating an EMR cluster"
)

keyname = template.add_parameter(Parameter(
    "KeyName",
    Description="Name of an existing EC2 KeyPair to enable SSH "
                "to the instances",
    Type=KEY_PAIR_NAME
))

subnet = template.add_parameter(Parameter(
    "Subnet",
    Description="Subnet ID for creating the EMR cluster",
    Type=SUBNET_ID
))

spot = template.add_parameter(Parameter(
    "SpotPrice",
    Description="Spot price (or use 0 for 'on demand' instance)",
    Type=NUMBER,
    Default="0.1"
))

withSpotPrice = "WithSpotPrice"
template.add_condition(withSpotPrice, Not(Equals(Ref(spot), "0")))

gcTimeRatio = template.add_parameter(Parameter(
    "GcTimeRatioValue",
    Description="Hadoop name node garbage collector time ratio",
    Type=NUMBER,
    Default="19"
))

# IAM roles required by EMR

emr_service_role = template.add_resource(iam.Role(
    'EMRServiceRole',
    AssumeRolePolicyDocument={
        "Statement": [{
            "Effect": "Allow",
            "Principal": {
                "Service": [
                    "elasticmapreduce.amazonaws.com"
                ]
            },
            "Action": ["sts:AssumeRole"]
        }]
    },
    ManagedPolicyArns=[
        'arn:aws:iam::aws:policy/service-role/AmazonElasticMapReduceRole'
    ]
))

emr_autoscaling_role = "EMR_AutoScaling_DefaultRole"

emr_job_flow_role = template.add_resource(iam.Role(
    "EMRJobFlowRole",
    AssumeRolePolicyDocument={
        "Statement": [{
            "Effect": "Allow",
            "Principal": {
                "Service": [
                    "ec2.amazonaws.com"
                ]
            },
            "Action": ["sts:AssumeRole"]
        }]
    },
    ManagedPolicyArns=[
        'arn:aws:iam::aws:policy/service-role/AmazonElasticMapReduceforEC2Role'
    ]
))

emr_instance_profile = template.add_resource(iam.InstanceProfile(
    "EMRInstanceProfile",
    Roles=[Ref(emr_job_flow_role)]
))

# EMR Cluster Resource

security_config = template.add_resource(emr.SecurityConfiguration(
    'EMRSecurityConfiguration',
    Name="EMRSampleClusterSecurityConfiguration",
    SecurityConfiguration=security_configuration,
))

cluster = template.add_resource(emr.Cluster(
    "EMRSampleCluster",
    Name="EMR Sample Cluster",
    ReleaseLabel='emr-4.4.0',
    SecurityConfiguration=Ref(security_config),
    BootstrapActions=[emr.BootstrapActionConfig(
        Name='Dummy bootstrap action',
        ScriptBootstrapAction=emr.ScriptBootstrapActionConfig(
            Path='file:/usr/share/aws/emr/scripts/install-hue',
            Args=["dummy", "parameter"]
        )
    )],
    Configurations=[
        emr.Configuration(
            Classification="core-site",
            ConfigurationProperties={
                'hadoop.security.groups.cache.secs': '250'
            }
        ),
        emr.Configuration(
            Classification="mapred-site",
            ConfigurationProperties={
                'mapred.tasktracker.map.tasks.maximum': '2',
                'mapreduce.map.sort.spill.percent': '90',
                'mapreduce.tasktracker.reduce.tasks.maximum': '5'
            }
        ),
        emr.Configuration(
            Classification="hadoop-env",
            Configurations=[
                emr.Configuration(
                    Classification="export",
                    ConfigurationProperties={
                        "HADOOP_DATANODE_HEAPSIZE": "2048",
                        "HADOOP_NAMENODE_OPTS": Join("", ["-XX:GCTimeRatio=",
                                                          Ref(gcTimeRatio)])
                    }
                )
            ]
        )
    ],
    JobFlowRole=Ref(emr_instance_profile),
    ServiceRole=Ref(emr_service_role),
    AutoScalingRole=emr_autoscaling_role,
    Instances=emr.JobFlowInstancesConfig(
        Ec2KeyName=Ref(keyname),
        Ec2SubnetId=Ref(subnet),
        MasterInstanceGroup=emr.InstanceGroupConfigProperty(
            Name="Master Instance",
            InstanceCount="1",
            InstanceType=M4_LARGE,
            Market="ON_DEMAND",
            AutoScalingPolicy=emr.AutoScalingPolicy(
                Constraints=emr.ScalingConstraints(
                    MinCapacity="1",
                    MaxCapacity="3"
                ),
                Rules=generate_rules("MasterAutoScalingPolicy")
            )
        ),
        CoreInstanceGroup=emr.InstanceGroupConfigProperty(
            Name="Core Instance",
            BidPrice=If(withSpotPrice, Ref(spot), Ref("AWS::NoValue")),
            Market=If(withSpotPrice, "SPOT", "ON_DEMAND"),
            AutoScalingPolicy=emr.AutoScalingPolicy(
                Constraints=emr.ScalingConstraints(
                    MinCapacity="1",
                    MaxCapacity="3"
                ),
                Rules=generate_rules("CoreAutoScalingPolicy"),
            ),
            EbsConfiguration=emr.EbsConfiguration(
                EbsBlockDeviceConfigs=[
                    emr.EbsBlockDeviceConfigs(
                        VolumeSpecification=emr.VolumeSpecification(
                            SizeInGB="10",
                            VolumeType="gp2"
                        ),
                        VolumesPerInstance="1"
                    )
                ],
                EbsOptimized="true"
            ),
            InstanceCount="1",
            InstanceType=M4_LARGE,
        )
    ),
    Applications=[
        emr.Application(Name="Hadoop"),
        emr.Application(Name="Hive"),
        emr.Application(Name="Mahout"),
        emr.Application(Name="Pig"),
        emr.Application(Name="Spark")
    ],
    VisibleToAllUsers="true",
    Tags=Tags(
        Name="EMR Sample Cluster"
    )
))

step = template.add_resource(emr.Step(
    'TestStep',
    Name="TestStep",
    ActionOnFailure='CONTINUE',
    HadoopJarStep=emr.HadoopJarStepConfig(
        Args=["5", "10"],
        Jar="s3://emr-cfn-test/hadoop-mapreduce-examples-2.6.0.jar",
        MainClass="pi",
        StepProperties=[
            emr.KeyValue('my.custom.property', 'my.custom.value')
        ]
    ),
    JobFlowId=Ref(cluster)
))

print(template.to_json())



# Converted from s3_processor located at:
# https://github.com/awslabs/serverless-application-model/blob/dbc54b5d0cd31bf5cebd16d765b74aee9eb34641/examples/2016-10-31/s3_processor/template.yaml

from troposphere import Template, Ref
from troposphere.s3 import Bucket
from troposphere.serverless import Function, S3Event

t = Template()

t.add_description(
    "A function is triggered off an upload to a bucket. It logs the content "
    "type of the uploaded object.")

t.add_transform('AWS::Serverless-2016-10-31')


s3_bucket = t.add_resource(
    Bucket("Bucket")
)

t.add_resource(
    Function(
        "ProcessorFunction",
        Handler='index.handler',
        Runtime='nodejs4.3',
        CodeUri='s3://<bucket>/s3_processor.zip',
        Policies='AmazonS3ReadOnlyAccess',
        Events={
            'PhotoUpload': S3Event(
                'PhotoUpload',
                Bucket=Ref(s3_bucket),
                Events=['s3:ObjectCreated:*']
            )
        }
    )
)

print(t.to_json())



#!/usr/bin/python

from troposphere import Template, Ref, Output, Parameter
from troposphere.dynamodb import (
    KeySchema,
    AttributeDefinition,
    ProvisionedThroughput,
    Projection,
    Table,
    GlobalSecondaryIndex
)

template = Template()

template.add_description("Create a dynamodb table with a global secondary "
                         "index")
# N.B. If you remove the provisioning section this works for
# LocalSecondaryIndexes aswell.

readunits = template.add_parameter(Parameter(
    "ReadCapacityUnits",
    Description="Provisioned read throughput",
    Type="Number",
    Default="10",
    MinValue="5",
    MaxValue="10000",
    ConstraintDescription="should be between 5 and 10000"
))

writeunits = template.add_parameter(Parameter(
    "WriteCapacityUnits",
    Description="Provisioned write throughput",
    Type="Number",
    Default="5",
    MinValue="5",
    MaxValue="10000",
    ConstraintDescription="should be between 5 and 10000"
))

tableIndexName = template.add_parameter(Parameter(
    "TableIndexName",
    Description="Table: Primary Key Field",
    Type="String",
    Default="id",
    AllowedPattern="[a-zA-Z0-9]*",
    MinLength="1",
    MaxLength="2048",
    ConstraintDescription="must contain only alphanumberic characters"
))

tableIndexDataType = template.add_parameter(Parameter(
    "TableIndexDataType",
    Description=" Table: Primary Key Data Type",
    Type="String",
    Default="S",
    AllowedPattern="[S|N|B]",
    MinLength="1",
    MaxLength="1",
    ConstraintDescription="S for string data, N for numeric data, or B for "
                          "binary data"
))

secondaryIndexHashName = template.add_parameter(Parameter(
    "SecondaryIndexHashName",
    Description="Secondary Index: Primary Key Field",
    Type="String",
    Default="tokenType",
    AllowedPattern="[a-zA-Z0-9]*",
    MinLength="1",
    MaxLength="2048",
    ConstraintDescription="must contain only alphanumberic characters"
))

secondaryIndexHashDataType = template.add_parameter(Parameter(
    "SecondaryIndexHashDataType",
    Description="Secondary Index: Primary Key Data Type",
    Type="String",
    Default="S",
    AllowedPattern="[S|N|B]",
    MinLength="1",
    MaxLength="1",
    ConstraintDescription="S for string data, N for numeric data, or B for "
                          "binary data"
))

secondaryIndexRangeName = template.add_parameter(Parameter(
    "refreshSecondaryIndexRangeName",
    Description="Secondary Index: Range Key Field",
    Type="String",
    Default="tokenUpdatedTime",
    AllowedPattern="[a-zA-Z0-9]*",
    MinLength="1",
    MaxLength="2048",
    ConstraintDescription="must contain only alphanumberic characters"
))

secondaryIndexRangeDataType = template.add_parameter(Parameter(
    "SecondaryIndexRangeDataType",
    Description="Secondary Index: Range Key Data Type",
    Type="String",
    Default="S",
    AllowedPattern="[S|N|B]",
    MinLength="1",
    MaxLength="1",
    ConstraintDescription="S for string data, N for numeric data, or B for "
                          "binary data"
))


GSITable = template.add_resource(Table(
    "GSITable",
    AttributeDefinitions=[
        AttributeDefinition(
            AttributeName=Ref(tableIndexName),
            AttributeType=Ref(tableIndexDataType)
        ),
        AttributeDefinition(
            AttributeName=Ref(secondaryIndexHashName),
            AttributeType=Ref(secondaryIndexHashDataType)
        ),
        AttributeDefinition(
            AttributeName=Ref(secondaryIndexRangeName),
            AttributeType=Ref(secondaryIndexRangeDataType)
        ),
        AttributeDefinition(
            AttributeName="non_key_attribute_0",
            AttributeType="S"
        ),
        AttributeDefinition(
            AttributeName="non_key_attribute_1",
            AttributeType="S"
        )
    ],
    KeySchema=[
        KeySchema(
            AttributeName=Ref(tableIndexName),
            KeyType="HASH"
        )
    ],
    ProvisionedThroughput=ProvisionedThroughput(
        ReadCapacityUnits=Ref(readunits),
        WriteCapacityUnits=Ref(writeunits)
    ),
    GlobalSecondaryIndexes=[
        GlobalSecondaryIndex(
            IndexName="SecondaryIndex",
            KeySchema=[
                KeySchema(
                    AttributeName=Ref(secondaryIndexHashName),
                    KeyType="HASH"
                ),
                KeySchema(
                    AttributeName=Ref(secondaryIndexRangeName),
                    KeyType="RANGE"
                )
            ],
            Projection=Projection(
                ProjectionType="INCLUDE",
                NonKeyAttributes=["non_key_attribute_0"]
            ),
            ProvisionedThroughput=ProvisionedThroughput(
                ReadCapacityUnits=Ref(readunits),
                WriteCapacityUnits=Ref(writeunits)
            )
        )
    ]
))

template.add_output(Output(
    "GSITable",
    Value=Ref(GSITable),
    Description="Table with a Global Secondary Index",
))


print(template.to_json())



from troposphere import Template
from troposphere.iot import (
    Certificate,
    Policy,
    PolicyPrincipalAttachment,
    Thing,
    ThingPrincipalAttachment,
    TopicRule,
    TopicRulePayload,
    Action,
    LambdaAction,
)


t = Template()

certificate = Certificate(
    'MyCertificate',
    CertificateSigningRequest='CSRParameter',
    Status='StatusParameter',
)

policy = Policy(
    'MyPolicy',
    PolicyDocument={'Version': '2012-10-17'},
    PolicyName='NameParameter',
)

policy_principal = PolicyPrincipalAttachment(
    'MyPolicyPrincipalAttachment',
    PolicyName='NameParameter',
    Principal='arn:aws:iot:ap-southeast-2:123456789012',
)

thing = Thing(
    'MyThing',
    AttributePayload={
        'Attributes': {
            'myAttributeA': 'MyAttributeValueA',
            'myAttributeB': 'MyAttributeValueB',
        }
    },
    ThingName='NameParameter',
)

thing_principal = ThingPrincipalAttachment(
    'MyThingPrincipalAttachment',
    ThingName='NameParameter',
    Principal='arn:aws:iot:ap-southeast-2:123456789012',
)

topic_rule = TopicRule(
    'MyTopicRule',
    RuleName='NameParameter',
    TopicRulePayload=TopicRulePayload(
        RuleDisabled=True,
        Sql='SELECT temp FROM SomeTopic WHERE temp > 60',
        Actions=[
            Action(
                Lambda=LambdaAction(
                    FunctionArn='arn',
                ),
            ),
        ],
    ),
)

t.add_resource(certificate)
t.add_resource(policy)
t.add_resource(policy_principal)
t.add_resource(thing)
t.add_resource(thing_principal)
t.add_resource(topic_rule)

print(t.to_json())



from .dynamodb import (
    AttributeDefinition,
    KeySchema,
    Key,
    ProvisionedThroughput,
    Projection,
    GlobalSecondaryIndex,
    LocalSecondaryIndex,
    StreamSpecification,
    Table,
    attribute_type_validator,
    key_type_validator,
    projection_type_validator,
)

import warnings

# Only way to make pyflakes shut up about unused imports
assert AttributeDefinition
assert KeySchema
assert Key
assert ProvisionedThroughput
assert Projection
assert GlobalSecondaryIndex
assert LocalSecondaryIndex
assert StreamSpecification
assert Table
assert attribute_type_validator
assert key_type_validator
assert projection_type_validator


warnings.warn("This module has replaced by troposphere.dynamodb. Please "
              "import that module instead, as troposphere.dynamodb2 will be "
              "removed soon.")



# Copyright (c) 2013, Bob Van Zant <bob@veznat.com>
# All rights reserved.
#
# See LICENSE file for full license.
import warnings

from . import AWSHelperFn, AWSObject, AWSProperty, Tags
from .validators import boolean, positive_integer, s3_bucket_name
from .validators import s3_transfer_acceleration_status

try:
    from awacs.aws import Policy

    policytypes = (dict, Policy)
except ImportError:
    policytypes = dict,

Private = "Private"
PublicRead = "PublicRead"
PublicReadWrite = "PublicReadWrite"
AuthenticatedRead = "AuthenticatedRead"
BucketOwnerRead = "BucketOwnerRead"
BucketOwnerFullControl = "BucketOwnerFullControl"
LogDeliveryWrite = "LogDeliveryWrite"


class CorsRules(AWSProperty):
    props = {
        'AllowedHeaders': ([basestring], False),
        'AllowedMethods': ([basestring], True),
        'AllowedOrigins': ([basestring], True),
        'ExposedHeaders': ([basestring], False),
        'Id': (basestring, False),
        'MaxAge': (positive_integer, False),
    }


class CorsConfiguration(AWSProperty):
    props = {
        'CorsRules': ([CorsRules], True),
    }


class VersioningConfiguration(AWSProperty):
    props = {
        'Status': (basestring, False),
    }


class AccelerateConfiguration(AWSProperty):
    props = {
        'AccelerationStatus': (s3_transfer_acceleration_status, True),
    }


class RedirectAllRequestsTo(AWSProperty):
    props = {
        'HostName': (basestring, True),
        'Protocol': (basestring, False),
    }


class RedirectRule(AWSProperty):
    props = {
        'HostName': (basestring, False),
        'HttpRedirectCode': (basestring, False),
        'Protocol': (basestring, False),
        'ReplaceKeyPrefixWith': (basestring, False),
        'ReplaceKeyWith': (basestring, False),
    }


class RoutingRuleCondition(AWSProperty):
    props = {
        'HttpErrorCodeReturnedEquals': (basestring, False),
        'KeyPrefixEquals': (basestring, False),
    }


class RoutingRule(AWSProperty):
    props = {
        'RedirectRule': (RedirectRule, True),
        'RoutingRuleCondition': (RoutingRuleCondition, False),
    }


class WebsiteConfiguration(AWSProperty):
    props = {
        'IndexDocument': (basestring, False),
        'ErrorDocument': (basestring, False),
        'RedirectAllRequestsTo': (RedirectAllRequestsTo, False),
        'RoutingRules': ([RoutingRule], False),
    }


class LifecycleRuleTransition(AWSProperty):
    props = {
        'StorageClass': (basestring, True),
        'TransitionDate': (basestring, False),
        'TransitionInDays': (positive_integer, False),
    }


class AbortIncompleteMultipartUpload(AWSProperty):
    props = {
        'DaysAfterInitiation': (positive_integer, True),
    }


class NoncurrentVersionTransition(AWSProperty):
    props = {
        'StorageClass': (basestring, True),
        'TransitionInDays': (positive_integer, True),
    }


class TagFilter(AWSProperty):
    props = {
        'Key': (basestring, True),
        'Value': (basestring, True),
    }


class LifecycleRule(AWSProperty):
    props = {
        'AbortIncompleteMultipartUpload':
            (AbortIncompleteMultipartUpload, False),
        'ExpirationDate': (basestring, False),
        'ExpirationInDays': (positive_integer, False),
        'Id': (basestring, False),
        'NoncurrentVersionExpirationInDays': (positive_integer, False),
        'NoncurrentVersionTransition': (NoncurrentVersionTransition, False),
        'NoncurrentVersionTransitions': ([NoncurrentVersionTransition], False),
        'Prefix': (basestring, False),
        'Status': (basestring, True),
        'TagFilters': ([TagFilter], False),
        'Transition': (LifecycleRuleTransition, False),
        'Transitions': ([LifecycleRuleTransition], False)
    }

    def validate(self):
        if 'Transition' in self.properties:
            if 'Transitions' not in self.properties:
                # aws moved from a single transition to a list of them
                # and deprecated 'Transition', so let's just move it to
                # the new property and not annoy the user.
                self.properties['Transitions'] = [
                    self.properties.pop('Transition')]
            else:
                raise ValueError(
                    'Cannot specify both "Transition" and "Transitions" '
                    'properties on S3 Bucket Lifecycle Rule. Please use '
                    '"Transitions" since the former has been deprecated.')

        if 'NoncurrentVersionTransition' in self.properties:
            if 'NoncurrentVersionTransitions' not in self.properties:
                warnings.warn(
                    'NoncurrentVersionTransition has been deprecated in '
                    'favour of NoncurrentVersionTransitions.'
                )
                # Translate the old transition format to the new format
                self.properties['NoncurrentVersionTransitions'] = [
                    self.properties.pop('NoncurrentVersionTransition')]
            else:
                raise ValueError(
                    'Cannot specify both "NoncurrentVersionTransition" and '
                    '"NoncurrentVersionTransitions" properties on S3 Bucket '
                    'Lifecycle Rule. Please use '
                    '"NoncurrentVersionTransitions" since the former has been '
                    'deprecated.')

        if 'ExpirationInDays' in self.properties and 'ExpirationDate' in \
                self.properties:
            raise ValueError(
                'Cannot specify both "ExpirationDate" and "ExpirationInDays"'
            )


class LifecycleConfiguration(AWSProperty):
    props = {
        'Rules': ([LifecycleRule], True),
    }


class LoggingConfiguration(AWSProperty):
    props = {
        'DestinationBucketName': (s3_bucket_name, False),
        'LogFilePrefix': (basestring, False),
    }


class Rules(AWSProperty):
    props = {
        'Name': (basestring, True),
        'Value': (basestring, True)
    }


class S3Key(AWSProperty):
    props = {
        'Rules': ([Rules], True)
    }


class Filter(AWSProperty):
    props = {
        'S3Key': (S3Key, True)
    }


class LambdaConfigurations(AWSProperty):
    props = {
        'Event': (basestring, True),
        'Filter': (Filter, False),
        'Function': (basestring, True),
    }


class QueueConfigurations(AWSProperty):
    props = {
        'Event': (basestring, True),
        'Filter': (Filter, False),
        'Queue': (basestring, True),
    }


class TopicConfigurations(AWSProperty):
    props = {
        'Event': (basestring, True),
        'Filter': (Filter, False),
        'Topic': (basestring, True),
    }


class MetricsConfiguration(AWSProperty):
    props = {
        'Id': (basestring, True),
        'Prefix': (basestring, False),
        'TagFilters': ([TagFilter], False),
    }


class NotificationConfiguration(AWSProperty):
    props = {
        'LambdaConfigurations': ([LambdaConfigurations], False),
        'QueueConfigurations': ([QueueConfigurations], False),
        'TopicConfigurations': ([TopicConfigurations], False),
    }


class AccessControlTranslation(AWSProperty):
    props = {
        'Owner': (basestring, True),
    }


class EncryptionConfiguration(AWSProperty):
    props = {
        'ReplicaKmsKeyID': (basestring, True),
    }


class ReplicationConfigurationRulesDestination(AWSProperty):
    props = {
        'AccessControlTranslation': (AccessControlTranslation, False),
        'Account': (basestring, False),
        'Bucket': (basestring, True),
        'EncryptionConfiguration': (EncryptionConfiguration, False),
        'StorageClass': (basestring, False),
    }


class SseKmsEncryptedObjects(AWSProperty):
    props = {
        'Status': (basestring, True),
    }


class SourceSelectionCriteria(AWSProperty):
    props = {
        'SseKmsEncryptedObjects': (SseKmsEncryptedObjects, True),
    }


class ReplicationConfigurationRules(AWSProperty):
    props = {
        'Destination': (ReplicationConfigurationRulesDestination, True),
        'Id': (basestring, False),
        'Prefix': (basestring, True),
        'SourceSelectionCriteria': (SourceSelectionCriteria, False),
        'Status': (basestring, True)
    }


class ReplicationConfiguration(AWSProperty):
    props = {
        'Role': (basestring, True),
        'Rules': ([ReplicationConfigurationRules], True)
    }


class Destination(AWSProperty):
    props = {
        'BucketAccountId': (basestring, False),
        'BucketArn': (basestring, True),
        'Format': (basestring, True),
        'Prefix': (basestring, False),
    }


class DataExport(AWSProperty):
    props = {
        'Destination': (Destination, True),
        'OutputSchemaVersion': (basestring, True),
    }


class StorageClassAnalysis(AWSProperty):
    props = {
        'DataExport': (DataExport, False),
    }


class AnalyticsConfiguration(AWSProperty):
    props = {
        'Id': (basestring, True),
        'Prefix': (basestring, False),
        'StorageClassAnalysis': (StorageClassAnalysis, True),
        'TagFilters': ([TagFilter], False),
    }


class ServerSideEncryptionByDefault(AWSProperty):
    props = {
        'KMSMasterKeyID': (basestring, False),
        'SSEAlgorithm': (basestring, True),
    }


class ServerSideEncryptionRule(AWSProperty):
    props = {
        'ServerSideEncryptionByDefault':
            (ServerSideEncryptionByDefault, False),
    }


class BucketEncryption(AWSProperty):
    props = {
        'ServerSideEncryptionConfiguration':
            ([ServerSideEncryptionRule], True),
    }


class InventoryConfiguration(AWSProperty):
    props = {
        'Destination': (Destination, True),
        'Enabled': (boolean, True),
        'Id': (basestring, True),
        'IncludedObjectVersions': (basestring, True),
        'OptionalFields': ([basestring], True),
        'Prefix': (basestring, False),
        'ScheduleFrequency': (basestring, True),
    }


class Bucket(AWSObject):
    resource_type = "AWS::S3::Bucket"

    props = {
        'AccessControl': (basestring, False),
        'AccelerateConfiguration': (AccelerateConfiguration, False),
        'AnalyticsConfigurations': ([AnalyticsConfiguration], False),
        'BucketEncryption': (BucketEncryption, False),
        'BucketName': (s3_bucket_name, False),
        'CorsConfiguration': (CorsConfiguration, False),
        'InventoryConfigurations': ([InventoryConfiguration], False),
        'LifecycleConfiguration': (LifecycleConfiguration, False),
        'LoggingConfiguration': (LoggingConfiguration, False),
        'MetricsConfigurations': ([MetricsConfiguration], False),
        'NotificationConfiguration': (NotificationConfiguration, False),
        'ReplicationConfiguration': (ReplicationConfiguration, False),
        'Tags': (Tags, False),
        'WebsiteConfiguration': (WebsiteConfiguration, False),
        'VersioningConfiguration': (VersioningConfiguration, False)
    }

    access_control_types = [
        Private,
        PublicRead,
        PublicReadWrite,
        AuthenticatedRead,
        BucketOwnerRead,
        BucketOwnerFullControl,
        LogDeliveryWrite,
    ]

    def validate(self):
        access_control = self.properties.get('AccessControl')
        if access_control is not None and \
                not isinstance(access_control, AWSHelperFn):
            if access_control not in self.access_control_types:
                raise ValueError('AccessControl must be one of "%s"' % (
                    ', '.join(self.access_control_types)))


class BucketPolicy(AWSObject):
    resource_type = "AWS::S3::BucketPolicy"

    props = {
        'Bucket': (basestring, True),
        'PolicyDocument': (policytypes, True),
    }



# Copyright (c) 2016-2017, troposphere project
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty
from .validators import boolean, positive_integer


def processor_type_validator(x):
    valid_types = ["Lambda"]
    if x not in valid_types:
        raise ValueError("Type must be one of: %s" %
                         ", ".join(valid_types))
    return x


def delivery_stream_type_validator(x):
    valid_types = ["DirectPut", "KinesisStreamAsSource"]
    if x not in valid_types:
        raise ValueError("DeliveryStreamType must be one of: %s" %
                         ", ".join(valid_types))
    return x


def index_rotation_period_validator(x):
    valid_types = ["NoRotation", "OneHour", "OneDay", "OneWeek", "OneMonth"]
    if x not in valid_types:
        raise ValueError("IndexRotationPeriod must be one of: %s" %
                         ", ".join(valid_types))
    return x


def s3_backup_mode_elastic_search_validator(x):
    valid_types = ["FailedDocumentsOnly", "AllDocuments"]
    if x not in valid_types:
        raise ValueError("S3BackupMode must be one of: %s" %
                         ", ".join(valid_types))
    return x


def s3_backup_mode_extended_s3_validator(x):
    valid_types = ["Disabled", "Enabled"]
    if x not in valid_types:
        raise ValueError("S3BackupMode must be one of: %s" %
                         ", ".join(valid_types))
    return x


class BufferingHints(AWSProperty):
    props = {
        'IntervalInSeconds': (positive_integer, True),
        'SizeInMBs': (positive_integer, True)
    }


class CloudWatchLoggingOptions(AWSProperty):
    props = {
        'Enabled': (boolean, False),
        'LogGroupName': (basestring, False),  # Conditional
        'LogStreamName': (basestring, False),  # Conditional
    }


class RetryOptions(AWSProperty):
    props = {
        'DurationInSeconds': (positive_integer, True),
    }


class KMSEncryptionConfig(AWSProperty):
    props = {
        'AWSKMSKeyARN': (basestring, True),
    }


class EncryptionConfiguration(AWSProperty):
    props = {
        'KMSEncryptionConfig': (KMSEncryptionConfig, False),
        'NoEncryptionConfig': (basestring, False),
    }


class S3Configuration(AWSProperty):
    props = {
        'BucketARN': (basestring, True),
        'BufferingHints': (BufferingHints, True),
        'CloudWatchLoggingOptions': (CloudWatchLoggingOptions, False),
        'CompressionFormat': (basestring, True),
        'EncryptionConfiguration': (EncryptionConfiguration, False),
        'Prefix': (basestring, True),
        'RoleARN': (basestring, True)
    }


class CopyCommand(AWSProperty):
    props = {
        'CopyOptions': (basestring, False),
        'DataTableColumns': (basestring, False),
        'DataTableName': (basestring, True),
    }


class ProcessorParameter(AWSProperty):
    props = {
        'ParameterName': (basestring, True),
        'ParameterValue': (basestring, True),
    }


class Processor(AWSProperty):
    props = {
        'Parameters': ([ProcessorParameter], True),
        'Type': (processor_type_validator, True),
    }


class ProcessingConfiguration(AWSProperty):
    props = {
        'Enabled': (boolean, True),
        'Processors': ([Processor], True),
    }


class ElasticsearchDestinationConfiguration(AWSProperty):
    props = {
        'BufferingHints': (BufferingHints, True),
        'CloudWatchLoggingOptions': (CloudWatchLoggingOptions, False),
        'DomainARN': (basestring, True),
        'IndexName': (basestring, True),
        'IndexRotationPeriod': (index_rotation_period_validator, True),
        'ProcessingConfiguration': (ProcessingConfiguration, False),
        'RetryOptions': (RetryOptions, False),
        'RoleARN': (basestring, True),
        'S3BackupMode': (s3_backup_mode_elastic_search_validator, True),
        'S3Configuration': (S3Configuration, False),
        'TypeName': (basestring, True),
    }


class RedshiftDestinationConfiguration(AWSProperty):
    props = {
        'CloudWatchLoggingOptions': (CloudWatchLoggingOptions, False),
        'ClusterJDBCURL': (basestring, True),
        'CopyCommand': (CopyCommand, True),
        'Password': (basestring, True),
        'ProcessingConfiguration': (ProcessingConfiguration, False),
        'RoleARN': (basestring, True),
        'S3Configuration': (S3Configuration, True),
        'Username': (basestring, True),
    }


class S3DestinationConfiguration(AWSProperty):
    props = {
        'BucketARN': (basestring, True),
        'BufferingHints': (BufferingHints, True),
        'CloudWatchLoggingOptions': (CloudWatchLoggingOptions, False),
        'CompressionFormat': (basestring, True),
        'EncryptionConfiguration': (EncryptionConfiguration, False),
        'Prefix': (basestring, True),
        'RoleARN': (basestring, True),
    }


class ExtendedS3DestinationConfiguration(AWSProperty):
    props = {
        'BucketARN': (basestring, True),
        'BufferingHints': (BufferingHints, True),
        'CloudWatchLoggingOptions': (CloudWatchLoggingOptions, False),
        'CompressionFormat': (basestring, True),
        'EncryptionConfiguration': (EncryptionConfiguration, False),
        'Prefix': (basestring, True),
        'ProcessingConfiguration': (ProcessingConfiguration, False),
        'RoleARN': (basestring, True),
        'S3BackupConfiguration': (S3DestinationConfiguration, False),
        'S3BackupMode': (s3_backup_mode_extended_s3_validator, False),
    }


class KinesisStreamSourceConfiguration(AWSProperty):
    props = {
        'KinesisStreamARN': (basestring, True),
        'RoleARN': (basestring, True)
    }


class SplunkRetryOptions(AWSProperty):
    props = {
        'DurationInSeconds': (positive_integer, True),
    }


class SplunkDestinationConfiguration(AWSProperty):
    props = {
        'CloudWatchLoggingOptions': (CloudWatchLoggingOptions, False),
        'HECAcknowledgmentTimeoutInSeconds': (positive_integer, False),
        'HECEndpoint': (basestring, True),
        'HECEndpointType': (basestring, True),
        'HECToken': (basestring, True),
        'ProcessingConfiguration': (ProcessingConfiguration, False),
        'RetryOptions': (SplunkRetryOptions, False),
        'S3BackupMode': (basestring, False),
        'S3Configuration': (S3DestinationConfiguration, True),
    }


class DeliveryStream(AWSObject):
    resource_type = "AWS::KinesisFirehose::DeliveryStream"

    props = {
        'DeliveryStreamName': (basestring, False),
        'DeliveryStreamType': (delivery_stream_type_validator, False),
        'ElasticsearchDestinationConfiguration': (ElasticsearchDestinationConfiguration, False),  # noqa
        'ExtendedS3DestinationConfiguration': (ExtendedS3DestinationConfiguration, False),  # noqa
        'KinesisStreamSourceConfiguration': (KinesisStreamSourceConfiguration, False),  # noqa
        'RedshiftDestinationConfiguration': (RedshiftDestinationConfiguration, False),  # noqa
        'S3DestinationConfiguration': (S3DestinationConfiguration, False),
        'SplunkDestinationConfiguration':
            (SplunkDestinationConfiguration, False),
    }



from troposphere import Base64, Join
from troposphere import Ref, Template
from troposphere.cloudformation import Init, InitConfig, InitFiles, InitFile
from troposphere.cloudformation import InitServices, InitService
from troposphere.iam import PolicyType
from troposphere.autoscaling import LaunchConfiguration
from troposphere.iam import Role
from troposphere.ecs import Cluster
from troposphere.autoscaling import AutoScalingGroup, Metadata
from troposphere.iam import InstanceProfile

t = Template()
t.add_version('2010-09-09')

PolicyEcr = t.add_resource(PolicyType(
    'PolicyEcr',
    PolicyName='EcrPolicy',
    PolicyDocument={'Version': '2012-10-17',
                    'Statement': [{'Action': ['ecr:GetAuthorizationToken'],
                                   'Resource': ['*'],
                                   'Effect': 'Allow'},
                                  {'Action': ['ecr:GetDownloadUrlForLayer',
                                              'ecr:BatchGetImage',
                                              'ecr:BatchCheckLayerAvailability'
                                              ],
                                   'Resource': [
                                   '*'],
                                   'Effect': 'Allow',
                                   'Sid': 'AllowPull'},
                                  ]},
    Roles=[Ref('EcsClusterRole')],
))

PolicyEcs = t.add_resource(PolicyType(
    'PolicyEcs',
    PolicyName='EcsPolicy',
    PolicyDocument={'Version': '2012-10-17',
                    'Statement': [
                        {'Action': ['ecs:CreateCluster',
                                    'ecs:RegisterContainerInstance',
                                    'ecs:DeregisterContainerInstance',
                                    'ecs:DiscoverPollEndpoint',
                                    'ecs:Submit*',
                                    'ecs:Poll',
                                    'ecs:StartTelemetrySession'],
                         'Resource': '*',
                         'Effect': 'Allow'}
                    ]},
    Roles=[Ref('EcsClusterRole')],
))

PolicyCloudwatch = t.add_resource(PolicyType(
    'PolicyCloudwatch',
    PolicyName='Cloudwatch',
    PolicyDocument={'Version': '2012-10-17',
                    'Statement': [{'Action': ['cloudwatch:*'], 'Resource': '*',
                                   'Effect': 'Allow'}]},
    Roles=[Ref('EcsClusterRole')],
))


ContainerInstances = t.add_resource(LaunchConfiguration(
    'ContainerInstances',
    Metadata=Metadata(
        Init({
            'config': InitConfig(
                files=InitFiles({
                    '/etc/cfn/cfn-hup.conf': InitFile(
                        content=Join('', ['[main]\n', 'stack=', Ref('AWS::StackId'),  # NOQA
                                          '\n', 'region=', Ref('AWS::Region'), '\n']),  # NOQA
                        mode='000400',
                        owner='root',
                        group='root'
                    ),
                    '/etc/cfn/hooks.d/cfn-auto-reloader.conf': InitFile(
                        content=Join('', ['[cfn-auto-reloader-hook]\n',
                                          'triggers=post.update\n',
                                          'path=Resources.ContainerInstances.Metadata.AWS::CloudFormation::Init\n',  # NOQA
                                          'action=/opt/aws/bin/cfn-init -v ', '--stack ', Ref(  # NOQA
                                              'AWS::StackName'), ' --resource ContainerInstances ', ' --region ', Ref('AWS::Region'), '\n',  # NOQA
                                          'runas=root\n']),
                        mode='000400',
                        owner='root',
                        group='root'
                    )},
                ),
                services=InitServices({
                    'cfn-hup': InitService(
                        ensureRunning='true',
                        enabled='true',
                        files=['/etc/cfn/cfn-hup.conf',
                               '/etc/cfn/hooks.d/cfn-auto-reloader.conf']
                    )}
                ),
                commands={
                    '01_add_instance_to_cluster': {'command': Join('',
                                                                   ['#!/bin/bash\n',  # NOQA
                                                                    'echo ECS_CLUSTER=',  # NOQA
                                                                    Ref('ECSCluster'),  # NOQA
                                                                    ' >> /etc/ecs/ecs.config'])},  # NOQA
                    '02_install_ssm_agent': {'command': Join('',
                                                             ['#!/bin/bash\n',
                                                              'yum -y update\n',  # NOQA
                                                              'curl https://amazon-ssm-eu-west-1.s3.amazonaws.com/latest/linux_amd64/amazon-ssm-agent.rpm -o amazon-ssm-agent.rpm\n',  # NOQA
                                                              'yum install -y amazon-ssm-agent.rpm'  # NOQA
                                                              ])}
                }
            )
        }
        ),
    ),
    UserData=Base64(Join('',
                         ['#!/bin/bash -xe\n',
                          'yum install -y aws-cfn-bootstrap\n',
                          '/opt/aws/bin/cfn-init -v ',
                          '         --stack ',
                          Ref('AWS::StackName'),
                             '         --resource ContainerInstances ',
                             '         --region ',
                             Ref('AWS::Region'),
                             '\n',
                             '/opt/aws/bin/cfn-signal -e $? ',
                             '         --stack ',
                             Ref('AWS::StackName'),
                             '         --resource ECSAutoScalingGroup ',
                             '         --region ',
                             Ref('AWS::Region'),
                             '\n'])),
    ImageId='ami-13f84d60',
    KeyName='yourkey',
    SecurityGroups=['sg-96114ef2'],
    IamInstanceProfile=Ref('EC2InstanceProfile'),
    InstanceType='t2.micro',
    AssociatePublicIpAddress='true',
))


ECSCluster = t.add_resource(Cluster(
    'ECSCluster',
))


ECSAutoScalingGroup = t.add_resource(AutoScalingGroup(
    'ECSAutoScalingGroup',
    DesiredCapacity='1',
    MinSize='1',
    MaxSize='1',
    VPCZoneIdentifier=['subnet-72849a0a', 'subnet-72849a08'],
    AvailabilityZones=['eu-west-1a', 'eu-west-1b'],
    LaunchConfigurationName=Ref('ContainerInstances'),
))

EC2InstanceProfile = t.add_resource(InstanceProfile(
    'EC2InstanceProfile',
    Path='/',
    Roles=[Ref('EcsClusterRole')],
))

EcsClusterRole = t.add_resource(Role(
    'EcsClusterRole',
    Path='/',
    ManagedPolicyArns=[
        'arn:aws:iam::aws:policy/service-role/AmazonEC2RoleforSSM'
    ],
    AssumeRolePolicyDocument={'Version': '2012-10-17',
                              'Statement': [{'Action': 'sts:AssumeRole',
                                             'Principal':
                                             {'Service': 'ec2.amazonaws.com'},
                                             'Effect': 'Allow',
                                             }]}
))

print(t.to_json())



from troposphere import Template
from troposphere.events import Rule, Target
from troposphere.awslambda import Function, Code
from troposphere import GetAtt, Join


t = Template()


# Create a Lambda function that will be mapped
code = [
    "var response = require('cfn-response');",
    "exports.handler = function(event, context) {",
    "   context.succeed('foobar!');",
    "   return 'foobar!';",
    "};",
]

# Create the Lambda function
foobar_function = t.add_resource(Function(
    "FoobarFunction",
    Code=Code(
        ZipFile=Join("", code)
    ),
    Handler="index.handler",
    Role=GetAtt("LambdaExecutionRole", "Arn"),
    Runtime="nodejs",
))

# Create the Event Target
foobar_target = Target(
    "FoobarTarget",
    Arn=GetAtt('FoobarFunction', 'Arn'),
    Id="FooBarFunction1"
)

# Create the Event Rule
rule = t.add_resource(Rule(
    "FoobarRule",
    EventPattern={
        "source": [
            "aws.ec2"
        ],
        "detail-type": [
            "EC2 Instance State-change Notification"
        ],
        "detail": {
            "state": [
                "stopping"
            ]
        }
    },
    Description="Foobar CloudWatch Event",
    State="ENABLED",
    Targets=[foobar_target]
))

print(t.to_json())



# Copyright (c) 2016, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSHelperFn, AWSObject, AWSProperty, Tags
from .validators import boolean, integer, positive_integer


class SourceAuth(AWSProperty):
    props = {
        'Resource': (basestring, False),
        'Type': (basestring, True),
    }

    def validate(self):
        valid_types = [
            'OAUTH'
        ]
        auth_types = self.properties.get('Type')
        if auth_types not in valid_types:
            raise ValueError('SourceAuth Type: must be one of %s' %
                             ','.join(valid_types))


class Artifacts(AWSProperty):
    props = {
        'Location': (basestring, False),
        'Name': (basestring, False),
        'NamespaceType': (basestring, False),
        'Packaging': (basestring, False),
        'Path': (basestring, False),
        'Type': (basestring, True),
    }

    def validate(self):
        valid_types = [
            'CODEPIPELINE',
            'NO_ARTIFACTS',
            'S3',
        ]
        artifact_type = self.properties.get('Type')
        if artifact_type not in valid_types:
            raise ValueError('Artifacts Type: must be one of %s' %
                             ','.join(valid_types))

        if artifact_type == 'S3':
            for required_property in ['Name', 'Location']:
                if not self.properties.get(required_property):
                    raise ValueError(
                        'Artifacts Type S3: requires %s to be set' %
                        required_property
                    )


class EnvironmentVariable(AWSProperty):
    props = {
        'Name': (basestring, True),
        'Type': (basestring, False),
        'Value': (basestring, True),
    }

    def validate(self):
        if 'Type' in self.properties:
            valid_types = [
                'PARAMETER_STORE',
                'PLAINTEXT',
            ]
            env_type = self.properties.get('Type')
            if env_type not in valid_types:
                raise ValueError(
                    'EnvironmentVariable Type: must be one of %s' %
                    ','.join(valid_types))


class Environment(AWSProperty):
    props = {
        'ComputeType': (basestring, True),
        'EnvironmentVariables': ((list, [EnvironmentVariable]), False),
        'Image': (basestring, True),
        'PrivilegedMode': (boolean, False),
        'Type': (basestring, True),
    }

    def validate(self):
        valid_types = [
            'LINUX_CONTAINER',
        ]
        env_type = self.properties.get('Type')
        if env_type not in valid_types:
            raise ValueError('Environment Type: must be one of %s' %
                             ','.join(valid_types))


class ProjectCache(AWSProperty):
    props = {
        'Location': (basestring, False),
        'Type': (basestring, True),
    }

    def validate(self):
        valid_types = [
            'NO_CACHE',
            'S3',
        ]
        cache_type = self.properties.get('Type')
        if cache_type not in valid_types:
            raise ValueError('ProjectCache Type: must be one of %s' %
                             ','.join(valid_types))


class Source(AWSProperty):
    props = {
        'Auth': (SourceAuth, False),
        'BuildSpec': (basestring, False),
        'GitCloneDepth': (positive_integer, False),
        'InsecureSsl': (boolean, False),
        'Location': (basestring, False),
        'Type': (basestring, True),
    }

    def validate(self):
        valid_types = [
            'CODECOMMIT',
            'CODEPIPELINE',
            'GITHUB',
            'GITHUB_ENTERPRISE',
            'S3',
        ]

        source_type = self.properties.get('Type')

        # Don't do additional checks if source_type can't
        # be determined (for example, being a Ref).
        if isinstance(source_type, AWSHelperFn):
            return

        if source_type not in valid_types:
            raise ValueError('Source Type: must be one of %s' %
                             ','.join(valid_types))

        location = self.properties.get('Location')
        if source_type is not 'CODEPIPELINE' and not location:
            raise ValueError(
                'Source Location: must be defined when type is %s' %
                source_type
                )

        auth = self.properties.get('Auth')
        if auth is not None and source_type is not 'GITHUB':
            raise ValueError("SourceAuth: must only be defined when using "
                             "'GITHUB' Source Type.")


class VpcConfig(AWSProperty):
    props = {
        'SecurityGroupIds': ([basestring], True),
        'Subnets': ([basestring], True),
        'VpcId': (basestring, True),
    }


class ProjectTriggers(AWSProperty):
    props = {
        'Webhook': (boolean, False),
    }


class Project(AWSObject):
    resource_type = "AWS::CodeBuild::Project"

    props = {
        'Artifacts': (Artifacts, True),
        'BadgeEnabled': (boolean, False),
        'Cache': (ProjectCache, False),
        'Description': (basestring, False),
        'EncryptionKey': (basestring, False),
        'Environment': (Environment, True),
        'Name': (basestring, True),
        'ServiceRole': (basestring, True),
        'Source': (Source, True),
        'Tags': (Tags, False),
        'TimeoutInMinutes': (integer, False),
        'Triggers': (ProjectTriggers, False),
        'VpcConfig': (VpcConfig, False),
    }



# Copyright (c) 2012-2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, If, Tags
from .validators import (
    elb_name, exactly_one, network_port,
    tg_healthcheck_port, integer
)


class LoadBalancerAttributes(AWSProperty):
    props = {
        'Key': (basestring, False),
        'Value': (basestring, False)
    }


class Certificate(AWSProperty):
    props = {
        'CertificateArn': (basestring, False)
    }


class Action(AWSProperty):
    props = {
        'TargetGroupArn': (basestring, True),
        'Type': (basestring, True)
    }


class Condition(AWSProperty):
    props = {
        'Field': (basestring, True),
        'Values': ([basestring], True)
    }


class Matcher(AWSProperty):
    props = {
        'HttpCode': (basestring, False)
    }


class SubnetMapping(AWSProperty):
    props = {
        'AllocationId': (basestring, True),
        'SubnetId': (basestring, True)
    }


class TargetGroupAttribute(AWSProperty):
    props = {
        'Key': (basestring, False),
        'Value': (basestring, False)
    }


class TargetDescription(AWSProperty):
    props = {
        'AvailabilityZone': (basestring, False),
        'Id': (basestring, True),
        'Port': (network_port, False)
    }


class Listener(AWSObject):
    resource_type = "AWS::ElasticLoadBalancingV2::Listener"

    props = {
        'Certificates': ([Certificate], False),
        'DefaultActions': ([Action], True),
        'LoadBalancerArn': (basestring, True),
        'Port': (network_port, True),
        'Protocol': (basestring, True),
        'SslPolicy': (basestring, False)
    }


class ListenerCertificate(AWSObject):
    resource_type = "AWS::ElasticLoadBalancingV2::ListenerCertificate"

    props = {
        'Certificates': ([Certificate], True),
        'ListenerArn': (basestring, True),
    }


class ListenerRule(AWSObject):
    resource_type = "AWS::ElasticLoadBalancingV2::ListenerRule"

    props = {
        'Actions': ([Action], True),
        'Conditions': ([Condition], True),
        'ListenerArn': (basestring, True),
        'Priority': (integer, True)
    }


TARGET_TYPE_INSTANCE = 'instance'
TARGET_TYPE_IP = 'ip'


class TargetGroup(AWSObject):
    resource_type = "AWS::ElasticLoadBalancingV2::TargetGroup"

    props = {
        'HealthCheckIntervalSeconds': (integer, False),
        'HealthCheckPath': (basestring, False),
        'HealthCheckPort': (tg_healthcheck_port, False),
        'HealthCheckProtocol': (basestring, False),
        'HealthCheckTimeoutSeconds': (integer, False),
        'HealthyThresholdCount': (integer, False),
        'Matcher': (Matcher, False),
        'Name': (basestring, False),
        'Port': (network_port, True),
        'Protocol': (basestring, True),
        'Tags': ((Tags, list), False),
        'TargetGroupAttributes': ([TargetGroupAttribute], False),
        'Targets': ([TargetDescription], False),
        'TargetType': (basestring, False),
        'UnhealthyThresholdCount': (integer, False),
        'VpcId': (basestring, True),
    }


class LoadBalancer(AWSObject):
    resource_type = "AWS::ElasticLoadBalancingV2::LoadBalancer"

    props = {
        'LoadBalancerAttributes': ([LoadBalancerAttributes], False),
        'Name': (elb_name, False),
        'Scheme': (basestring, False),
        'IpAddressType': (basestring, False),
        'SecurityGroups': (list, False),
        'SubnetMappings': ([SubnetMapping], False),
        'Subnets': (list, False),
        'Tags': ((Tags, list), False),
        'Type': (basestring, False),
    }

    def validate(self):
        conds = [
            'SubnetMappings',
            'Subnets',
        ]

        def check_if(names, props):
            validated = []
            for name in names:
                validated.append(name in props and isinstance(props[name], If))
            return all(validated)

        if check_if(conds, self.properties):
            return

        exactly_one(self.__class__.__name__, self.properties, conds)



# Copyright (c) 2012-2018, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty
from .validators import boolean


class DimensionConfiguration(AWSProperty):
    props = {
        'DefaultDimensionValue': (basestring, True),
        'DimensionName': (basestring, True),
        'DimensionValueSource': (basestring, True),
    }


class CloudWatchDestination(AWSProperty):
    props = {
        'DimensionConfigurations': ([DimensionConfiguration], False),
    }


class KinesisFirehoseDestination(AWSProperty):
    props = {
        'DeliveryStreamARN': (basestring, True),
        'IAMRoleARN': (basestring, True),
    }


class EventDestination(AWSProperty):
    props = {
        'CloudWatchDestination': (CloudWatchDestination, False),
        'Enabled': (boolean, False),
        'KinesisFirehoseDestination': (KinesisFirehoseDestination, False),
        'MatchingEventTypes': ([basestring], True),
        'Name': (basestring, False),
    }


class ConfigurationSetEventDestination(AWSObject):
    resource_type = "AWS::SES::ConfigurationSetEventDestination"

    props = {
        'ConfigurationSetName': (basestring, True),
        'EventDestination': (EventDestination, True),
    }


class ConfigurationSet(AWSObject):
    resource_type = "AWS::SES::ConfigurationSet"

    props = {
        'Name': (basestring, False),
    }


class IpFilter(AWSProperty):
    props = {
        'Cidr': (basestring, True),
        'Policy': (basestring, True),
    }


class Filter(AWSProperty):
    props = {
        'IpFilter': (IpFilter, True),
        'Name': (basestring, False),
    }


class ReceiptFilter(AWSObject):
    resource_type = "AWS::SES::ReceiptFilter"

    props = {
        'Filter': (Filter, True),
    }


class ReceiptRuleSet(AWSObject):
    resource_type = "AWS::SES::ReceiptRuleSet"

    props = {
        'RuleSetName': (basestring, False),
    }


class AddHeaderAction(AWSProperty):
    props = {
        'HeaderName': (basestring, True),
        'HeaderValue': (basestring, True),
    }


class BounceAction(AWSProperty):
    props = {
        'Message': (basestring, True),
        'Sender': (basestring, True),
        'SmtpReplyCode': (basestring, True),
        'StatusCode': (basestring, False),
        'TopicArn': (basestring, False),
    }


class LambdaAction(AWSProperty):
    props = {
        'FunctionArn': (basestring, True),
        'InvocationType': (basestring, False),
        'TopicArn': (basestring, False),
    }


class S3Action(AWSProperty):
    props = {
        'BucketName': (basestring, True),
        'KmsKeyArn': (basestring, False),
        'ObjectKeyPrefix': (basestring, False),
        'TopicArn': (basestring, False),
    }


class SNSAction(AWSProperty):
    props = {
        'Encoding': (basestring, False),
        'TopicArn': (basestring, False),
    }


class StopAction(AWSProperty):
    props = {
        'Scope': (basestring, True),
        'TopicArn': (basestring, False),
    }


class WorkmailAction(AWSProperty):
    props = {
        'OrganizationArn': (basestring, True),
        'TopicArn': (basestring, False),
    }


class Action(AWSProperty):
    props = {
        'AddHeaderAction': (AddHeaderAction, False),
        'BounceAction': (BounceAction, False),
        'LambdaAction': (LambdaAction, False),
        'S3Action': (S3Action, False),
        'SNSAction': (SNSAction, False),
        'StopAction': (StopAction, False),
        'WorkmailAction': (WorkmailAction, False),
    }


class Rule(AWSProperty):
    props = {
        'Actions': ([Action], False),
        'Enabled': (boolean, False),
        'Name': (basestring, False),
        'Recipients': ([basestring], False),
        'ScanEnabled': (boolean, False),
        'TlsPolicy': (basestring, False),
    }


class ReceiptRule(AWSObject):
    resource_type = "AWS::SES::ReceiptRule"

    props = {
        'After': (basestring, False),
        'Rule': (Rule, True),
        'RuleSetName': (basestring, True),
    }


class EmailTemplate(AWSProperty):
    props = {
        'HtmlPart': (basestring, False),
        'SubjectPart': (basestring, False),
        'TemplateName': (basestring, False),
        'TextPart': (basestring, False),
    }


class Template(AWSObject):
    resource_type = "AWS::SES::Template"

    props = {
        'Template': (EmailTemplate, False),
    }



"""Converted from CloudTrail example."""
"""http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloudtrail-trail.html."""  # NOQA

from troposphere import GetAtt, Join
from troposphere import Parameter, Ref, Template
from troposphere.s3 import Bucket
from troposphere.sns import Topic, Subscription
from troposphere.sns import TopicPolicy
from troposphere.s3 import BucketPolicy
from troposphere.cloudtrail import Trail


t = Template()

t.add_version("2010-09-09")

OperatorEmail = t.add_parameter(Parameter(
    "OperatorEmail",
    Type="String",
    Description="Email address to notify when new logs are published.",
))

S3Bucket = t.add_resource(Bucket(
    "S3Bucket",
    DeletionPolicy="Retain"
))

Topic = t.add_resource(Topic(
    "Topic",
    Subscription=[
        Subscription(
            Endpoint=Ref(OperatorEmail),
            Protocol="email",
        ),
    ],
))

TopicPolicy = t.add_resource(TopicPolicy(
    "TopicPolicy",
    Topics=[Ref(Topic)],
    PolicyDocument={
        "Version": "2008-10-17", "Statement": [{
            "Action": "SNS:Publish",
            "Principal": {
                "Service": "cloudtrail.amazonaws.com"
            },
            "Resource": "*", "Effect": "Allow", "Sid": "AWSCloudTrailSNSPolicy"
        }]
    },
))

BucketPolicy = t.add_resource(BucketPolicy(
    "BucketPolicy",
    PolicyDocument={
        "Version": "2012-10-17",
        "Statement": [{
            "Action": "s3:GetBucketAcl",
            "Principal": {
                "Service": "cloudtrail.amazonaws.com"
            },
            "Resource": Join("", [
                "arn:aws:s3:::", Ref(S3Bucket)
            ]),
            "Effect": "Allow",
            "Sid": "AWSCloudTrailAclCheck"
        }, {
            "Action": "s3:PutObject",
            "Principal": {
                "Service": "cloudtrail.amazonaws.com"
            },
            "Resource": Join("", [
                "arn:aws:s3:::",
                Ref(S3Bucket),
                "/AWSLogs/",
                Ref("AWS::AccountId"),
                "/*"
            ]),
            "Effect": "Allow",
            "Sid": "AWSCloudTrailWrite",
            "Condition": {
                "StringEquals": {"s3:x-amz-acl": "bucket-owner-full-control"}
            }
        }]
    },
    Bucket=Ref(S3Bucket),
))

myTrail = t.add_resource(Trail(
    "myTrail",
    IsLogging=True,
    S3BucketName=Ref(S3Bucket),
    SnsTopicName=GetAtt(Topic, "TopicName"),
    DependsOn=["BucketPolicy", "TopicPolicy"],
))

print(t.to_json())



from troposphere import Ref, Template, Output
from troposphere.apigateway import RestApi, Method
from troposphere.apigateway import Resource, MethodResponse
from troposphere.apigateway import Integration, IntegrationResponse
from troposphere.apigateway import Deployment, Stage, ApiStage
from troposphere.apigateway import UsagePlan, QuotaSettings, ThrottleSettings
from troposphere.apigateway import ApiKey, StageKey, UsagePlanKey
from troposphere.iam import Role, Policy
from troposphere.awslambda import Function, Code
from troposphere import GetAtt, Join


t = Template()

# Create the Api Gateway
rest_api = t.add_resource(RestApi(
    "ExampleApi",
    Name="ExampleApi"
))

# Create a Lambda function that will be mapped
code = [
    "var response = require('cfn-response');",
    "exports.handler = function(event, context) {",
    "   context.succeed('foobar!');",
    "   return 'foobar!';",
    "};",
]

# Create a role for the lambda function
t.add_resource(Role(
    "LambdaExecutionRole",
    Path="/",
    Policies=[Policy(
        PolicyName="root",
        PolicyDocument={
            "Version": "2012-10-17",
            "Statement": [{
                "Action": ["logs:*"],
                "Resource": "arn:aws:logs:*:*:*",
                "Effect": "Allow"
            }, {
                "Action": ["lambda:*"],
                "Resource": "*",
                "Effect": "Allow"
            }]
        })],
    AssumeRolePolicyDocument={"Version": "2012-10-17", "Statement": [
        {
            "Action": ["sts:AssumeRole"],
            "Effect": "Allow",
            "Principal": {
                "Service": [
                    "lambda.amazonaws.com",
                    "apigateway.amazonaws.com"
                ]
            }
        }
    ]},
))

# Create the Lambda function
foobar_function = t.add_resource(Function(
    "FoobarFunction",
    Code=Code(
        ZipFile=Join("", code)
    ),
    Handler="index.handler",
    Role=GetAtt("LambdaExecutionRole", "Arn"),
    Runtime="nodejs4.3",
))

# Create a resource to map the lambda function to
resource = t.add_resource(Resource(
    "FoobarResource",
    RestApiId=Ref(rest_api),
    PathPart="foobar",
    ParentId=GetAtt("ExampleApi", "RootResourceId"),
))

# Create a Lambda API method for the Lambda resource
method = t.add_resource(Method(
    "LambdaMethod",
    DependsOn='FoobarFunction',
    RestApiId=Ref(rest_api),
    AuthorizationType="NONE",
    ResourceId=Ref(resource),
    HttpMethod="GET",
    Integration=Integration(
        Credentials=GetAtt("LambdaExecutionRole", "Arn"),
        Type="AWS",
        IntegrationHttpMethod='POST',
        IntegrationResponses=[
            IntegrationResponse(
                StatusCode='200'
            )
        ],
        Uri=Join("", [
            "arn:aws:apigateway:eu-west-1:lambda:path/2015-03-31/functions/",
            GetAtt("FoobarFunction", "Arn"),
            "/invocations"
        ])
    ),
    MethodResponses=[
        MethodResponse(
            "CatResponse",
            StatusCode='200'
        )
    ]
))

# Create a deployment
stage_name = 'v1'

deployment = t.add_resource(Deployment(
    "%sDeployment" % stage_name,
    DependsOn="LambdaMethod",
    RestApiId=Ref(rest_api),
))

stage = t.add_resource(Stage(
    '%sStage' % stage_name,
    StageName=stage_name,
    RestApiId=Ref(rest_api),
    DeploymentId=Ref(deployment)
))

key = t.add_resource(ApiKey(
    "ApiKey",
    StageKeys=[StageKey(
        RestApiId=Ref(rest_api),
        StageName=Ref(stage)
    )]
))

# Create an API usage plan
usagePlan = t.add_resource(UsagePlan(
    "ExampleUsagePlan",
    UsagePlanName="ExampleUsagePlan",
    Description="Example usage plan",
    Quota=QuotaSettings(
        Limit=50000,
        Period="MONTH"
    ),
    Throttle=ThrottleSettings(
        BurstLimit=500,
        RateLimit=5000
    ),
    ApiStages=[
        ApiStage(
            ApiId=Ref(rest_api),
            Stage=Ref(stage)
        )]
))

# tie the usage plan and key together
usagePlanKey = t.add_resource(UsagePlanKey(
    "ExampleUsagePlanKey",
    KeyId=Ref(key),
    KeyType="API_KEY",
    UsagePlanId=Ref(usagePlan)
))

# Add the deployment endpoint as an output
t.add_output([
    Output(
        "ApiEndpoint",
        Value=Join("", [
            "https://",
            Ref(rest_api),
            ".execute-api.eu-west-1.amazonaws.com/",
            stage_name
        ]),
        Description="Endpoint for this stage of the api"
    ),
    Output(
        "ApiKey",
        Value=Ref(key),
        Description="API key"
    ),
])


print(t.to_json())



# Converted from IAM_Users_Groups_and_Policies.template located at:
# http://aws.amazon.com/cloudformation/aws-cloudformation-templates/

from troposphere import GetAtt, Ref, Template
from troposphere.iam import LoginProfile, Policy, User
import awacs
import awacs.aws
import awacs.sns
import awacs.sqs


t = Template()

t.add_resource(User(
    "myuser",
    Path="/",
    LoginProfile=LoginProfile(Password="myP@ssW0rd"),
    Policies=[
        Policy(
            PolicyName="giveaccesstoqueueonly",
            PolicyDocument=awacs.aws.Policy(
                Statement=[
                    awacs.aws.Statement(
                        Effect=awacs.aws.Allow,
                        Action=[awacs.aws.Action("sqs", "*")],
                        Resource=[GetAtt("myqueue", "Arn")],
                    ),
                    awacs.aws.Statement(
                        Effect=awacs.aws.Deny,
                        Action=[awacs.aws.Action("sqs", "*")],
                        NotResource=[GetAtt("myqueue", "Arn")],
                    ),
                ],
            )
        ),
        Policy(
            PolicyName="giveaccesstotopiconly",
            PolicyDocument=awacs.aws.Policy(
                Statement=[
                    awacs.aws.Statement(
                        Effect=awacs.aws.Allow,
                        Action=[awacs.aws.Action("sns", "*")],
                        Resource=[Ref("mytopic")],
                    ),
                    awacs.aws.Statement(
                        Effect=awacs.aws.Deny,
                        Action=[awacs.aws.Action("sns", "*")],
                        NotResource=[Ref("mytopic")],
                    ),
                ],
            )
        ),
    ]
))

print(t.to_json())



# Copyright (c) 2012-2015, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSProperty, AWSObject, Tags
from .validators import boolean, integer, integer_range, positive_integer

VALID_VOLUME_TYPES = ('standard', 'gp2', 'io1')

try:
    from awacs.aws import Policy
    policytypes = (dict, Policy)
except ImportError:
    policytypes = dict,


def validate_volume_type(volume_type):
    """Validate VolumeType for ElasticsearchDomain"""
    if volume_type not in VALID_VOLUME_TYPES:
        raise ValueError("Elasticsearch Domain VolumeType must be one of: %s" %
                         ", ".join(VALID_VOLUME_TYPES))
    return volume_type


class EBSOptions(AWSProperty):
    props = {
        'EBSEnabled': (boolean, False),
        'Iops': (positive_integer, False),
        'VolumeSize': (integer, False),
        'VolumeType': (validate_volume_type, False)
    }

    def validate(self):
        volume_type = self.properties.get('VolumeType')
        iops = self.properties.get('Iops')
        if volume_type == 'io1' and not iops:
            raise ValueError("Must specify Iops if VolumeType is 'io1'.")


class ElasticsearchClusterConfig(AWSProperty):
    props = {
        'DedicatedMasterCount': (integer, False),
        'DedicatedMasterEnabled': (boolean, False),
        'DedicatedMasterType': (basestring, False),
        'InstanceCount': (integer, False),
        'InstanceType': (basestring, False),
        'ZoneAwarenessEnabled': (boolean, False)
    }


class EncryptionAtRestOptions(AWSProperty):
    props = {
        'Enabled': (boolean, False),
        'KmsKeyId': (basestring, False),
    }


class SnapshotOptions(AWSProperty):
    props = {
        'AutomatedSnapshotStartHour': (integer_range(0, 23), False)
    }


class VPCOptions(AWSProperty):
    props = {
        "SecurityGroupIds": ([basestring], False),
        "SubnetIds": ([basestring], False)
    }


class Domain(AWSObject):
    resource_type = "AWS::Elasticsearch::Domain"

    props = {
        'AccessPolicies': (policytypes, False),
        'AdvancedOptions': (dict, False),
        'DomainName': (basestring, False),
        'EBSOptions': (EBSOptions, False),
        'ElasticsearchClusterConfig': (ElasticsearchClusterConfig, False),
        'ElasticsearchVersion': (basestring, False),
        'EncryptionAtRestOptions': (EncryptionAtRestOptions, False),
        'SnapshotOptions': (SnapshotOptions, False),
        'Tags': ((Tags, list), False),
        'VPCOptions': (VPCOptions, False)
    }


# Backward compatibility
ElasticsearchDomain = Domain



# Copyright (c) 2015, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty
from .validators import boolean, exactly_one, positive_integer


KEY_ONLY = "KEY_ONLY"
VALUE_ONLY = "VALUE_ONLY"
KEY_AND_VALUE = "KEY_AND_VALUE"


class GitHubLocation(AWSProperty):
    props = {
        'CommitId': (basestring, True),
        'Repository': (basestring, True),
    }


class S3Location(AWSProperty):
    props = {
        'Bucket': (basestring, True),
        'BundleType': (basestring, True),
        'ETag': (basestring, False),
        'Key': (basestring, True),
        'Version': (basestring, False),
    }


class Revision(AWSProperty):
    props = {
        'GitHubLocation': (GitHubLocation, False),
        'RevisionType': (basestring, False),
        'S3Location': (S3Location, False),
    }


def deployment_option_validator(x):
    valid_values = ['WITH_TRAFFIC_CONTROL', 'WITHOUT_TRAFFIC_CONTROL']
    if x not in valid_values:
        raise ValueError("Deployment Option value must be one of: %s" %
                         ', '.join(valid_values))
    return x


def deployment_type_validator(x):
    valid_values = ['IN_PLACE', 'BLUE_GREEN']
    if x not in valid_values:
        raise ValueError("Deployment Type value must be one of: %s" %
                         ', '.join(valid_values))
    return x


class AutoRollbackConfiguration(AWSProperty):
    props = {
        'Enabled': (bool, False),
        'Events': ([basestring], False)
    }


class Deployment(AWSProperty):
    props = {
        'Description': (basestring, False),
        'IgnoreApplicationStopFailures': (bool, False),
        'Revision': (Revision, True),
    }


class DeploymentStyle(AWSProperty):
    props = {
        'DeploymentOption': (deployment_option_validator, False),
        'DeploymentType': (deployment_type_validator, False),
    }


class Ec2TagFilters(AWSProperty):
    props = {
        'Key': (basestring, False),
        'Type': (basestring, False),
        'Value': (basestring, False),
    }


class ElbInfoList(AWSProperty):
    props = {
        'Name': (basestring, False)
    }


class TargetGroupInfoList(AWSProperty):
    props = {
        'Name': (basestring, False)
    }


class LoadBalancerInfo(AWSProperty):
    props = {
        'ElbInfoList': ([ElbInfoList], False),
        'TargetGroupInfoList': ([TargetGroupInfoList], False),
    }

    def validate(self):
        conds = [
            'ElbInfoList',
            'TargetGroupInfoList',
        ]
        exactly_one(self.__class__.__name__, self.properties, conds)


class OnPremisesInstanceTagFilters(AWSProperty):
    props = {
        'Key': (basestring, False),
        'Type': (basestring, False),
        'Value': (basestring, False),
    }


class MinimumHealthyHosts(AWSProperty):
    props = {
        'Type': (basestring, False),
        'Value': (positive_integer, False),
    }


class Application(AWSObject):
    resource_type = "AWS::CodeDeploy::Application"

    props = {
        'ApplicationName': (basestring, False),
        'ComputePlatform': (basestring, False),
    }


class DeploymentConfig(AWSObject):
    resource_type = "AWS::CodeDeploy::DeploymentConfig"

    props = {
        'DeploymentConfigName': (basestring, False),
        'MinimumHealthyHosts': (MinimumHealthyHosts, False),
    }


class Alarm(AWSProperty):
    props = {
        'Name': (basestring, False),
    }


class AlarmConfiguration(AWSProperty):
    props = {
        'Alarms': ([Alarm], False),
        'Enabled': (boolean, False),
        'IgnorePollAlarmFailure': (boolean, False),
    }


class TriggerConfig(AWSProperty):
    props = {
        'TriggerEvents': ([basestring], False),
        'TriggerName': (basestring, False),
        'TriggerTargetArn': (basestring, False),
    }


class DeploymentGroup(AWSObject):
    resource_type = "AWS::CodeDeploy::DeploymentGroup"

    props = {
        'AlarmConfiguration': (AlarmConfiguration, False),
        'ApplicationName': (basestring, True),
        'AutoRollbackConfiguration': (AutoRollbackConfiguration, False),
        'AutoScalingGroups': ([basestring], False),
        'Deployment': (Deployment, False),
        'DeploymentConfigName': (basestring, False),
        'DeploymentGroupName': (basestring, False),
        'DeploymentStyle': (DeploymentStyle, False),
        'Ec2TagFilters': ([Ec2TagFilters], False),
        'LoadBalancerInfo': (LoadBalancerInfo, False),
        'OnPremisesInstanceTagFilters': (OnPremisesInstanceTagFilters, False),
        'ServiceRoleArn': (basestring, True),
        'TriggerConfigurations': ([TriggerConfig], False),
    }



# Copyright (c) 2012-2017, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, Tags


class Cluster(AWSObject):
    resource_type = "AWS::DAX::Cluster"

    props = {
        'AvailabilityZones': (basestring, False),
        'ClusterName': (basestring, False),
        'Description': (basestring, False),
        'IAMRoleARN': (basestring, True),
        'NodeType': (basestring, True),
        'NotificationTopicARN': (basestring, False),
        'ParameterGroupName': (basestring, False),
        'PreferredMaintenanceWindow': (basestring, False),
        'ReplicationFactor': (basestring, True),
        'SecurityGroupIds': ([basestring], False),
        'SubnetGroupName': (basestring, True),
        'Tags': (Tags, False),
    }


class ParameterGroup(AWSObject):
    resource_type = "AWS::DAX::ParameterGroup"

    props = {
        'Description': (basestring, False),
        'ParameterGroupName': (basestring, False),
        'ParameterNameValues': (dict, False),
    }


class SubnetGroup(AWSObject):
    resource_type = "AWS::DAX::SubnetGroup"

    props = {
        'Description': (basestring, False),
        'SubnetGroupName': (basestring, False),
        'SubnetIds': ([basestring], False),
    }



# Example Network with a NAT Gateway

from troposphere import Output, Ref, Template, Parameter, GetAtt
from troposphere import ec2


t = Template()

t.add_description(
    "AWS CloudFormation Sample Template NatGateway: Sample template showing "
    "how to create a public NAT gateway. "
    "**WARNING** This template creates an Amazon NAT gateway. "
    "You will be billed for the AWS resources used if you create "
    "a stack from this template.")

vpc_cidr = t.add_parameter(Parameter(
    'VPCCIDR',
    Default='172.18.0.0/16',
    Description='The IP address space for this VPC, in CIDR notation',
    Type='String',
))

public_subnet = t.add_parameter(Parameter(
    'PublicSubnetCidr',
    Type='String',
    Description='Public Subnet CIDR',
    Default='172.18.0.0/22',
))

private_subnet = t.add_parameter(Parameter(
    'PrivateSubnetCidr',
    Type='String',
    Description='Public Subnet CIDR',
    Default='172.18.32.0/21',
))

vpc = t.add_resource(ec2.VPC(
    'VPC',
    CidrBlock=Ref(vpc_cidr),
))

public_net = t.add_resource(ec2.Subnet(
    'PublicSubnet',
    CidrBlock=Ref(public_subnet),
    MapPublicIpOnLaunch=True,
    VpcId=Ref(vpc),
))

private_net = t.add_resource(ec2.Subnet(
    'PrivateSubnet',
    CidrBlock=Ref(private_subnet),
    MapPublicIpOnLaunch=False,
    VpcId=Ref(vpc),
))

igw = t.add_resource(ec2.InternetGateway('InternetGateway',))

net_gw_vpc_attachment = t.add_resource(ec2.VPCGatewayAttachment(
    "NatAttachment",
    VpcId=Ref(vpc),
    InternetGatewayId=Ref(igw),
))

private_route_table = t.add_resource(ec2.RouteTable(
    'PrivateRouteTable',
    VpcId=Ref(vpc),
))

public_route_table = t.add_resource(ec2.RouteTable(
    'PublicRouteTable',
    VpcId=Ref(vpc),
))

public_route_association = t.add_resource(ec2.SubnetRouteTableAssociation(
    'PublicRouteAssociation',
    SubnetId=Ref(public_net),
    RouteTableId=Ref(public_route_table),
))

default_public_route = t.add_resource(ec2.Route(
    'PublicDefaultRoute',
    RouteTableId=Ref(public_route_table),
    DestinationCidrBlock='0.0.0.0/0',
    GatewayId=Ref(igw),
))

private_route_association = t.add_resource(ec2.SubnetRouteTableAssociation(
    'PrivateRouteAssociation',
    SubnetId=Ref(private_net),
    RouteTableId=Ref(private_route_table),
))

nat_eip = t.add_resource(ec2.EIP(
    'NatEip',
    Domain="vpc",
))

nat = t.add_resource(ec2.NatGateway(
    'Nat',
    AllocationId=GetAtt(nat_eip, 'AllocationId'),
    SubnetId=Ref(public_net),
))

t.add_resource(ec2.Route(
    'NatRoute',
    RouteTableId=Ref(private_route_table),
    DestinationCidrBlock='0.0.0.0/0',
    NatGatewayId=Ref(nat),
))

t.add_output(Output(
    'VPCId',
    Value=Ref(vpc),
    Description='VPC Id'
))

t.add_output(Output(
    'NatEip',
    Value=Ref(nat_eip),
    Description='Nat Elastic IP',
))

print(t.to_json())



# Copyright (c) 2012-2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import boolean, integer, positive_integer, network_port


class Cookies(AWSProperty):
    props = {
        'Forward': (basestring, True),
        'WhitelistedNames': ([basestring], False),
    }


class ForwardedValues(AWSProperty):
    props = {
        'Cookies': (Cookies, False),
        'Headers': ([basestring], False),
        'QueryString': (boolean, True),
        'QueryStringCacheKeys': ([basestring], False),
    }


class LambdaFunctionAssociation(AWSProperty):
    props = {
        'EventType': (basestring, False),
        'LambdaFunctionARN': (basestring, False),
    }


class CacheBehavior(AWSProperty):
    props = {
        'AllowedMethods': ([basestring], False),
        'CachedMethods': ([basestring], False),
        'Compress': (boolean, False),
        'DefaultTTL': (integer, False),
        'ForwardedValues': (ForwardedValues, True),
        'LambdaFunctionAssociations': ([LambdaFunctionAssociation], False),
        'MaxTTL': (integer, False),
        'MinTTL': (integer, False),
        'PathPattern': (basestring, True),
        'SmoothStreaming': (boolean, False),
        'TargetOriginId': (basestring, True),
        'TrustedSigners': ([basestring], False),
        'ViewerProtocolPolicy': (basestring, True),
    }


class DefaultCacheBehavior(AWSProperty):
    props = {
        'AllowedMethods': ([basestring], False),
        'CachedMethods': ([basestring], False),
        'Compress': (boolean, False),
        'DefaultTTL': (integer, False),
        'ForwardedValues': (ForwardedValues, True),
        'LambdaFunctionAssociations': ([LambdaFunctionAssociation], False),
        'MaxTTL': (integer, False),
        'MinTTL': (integer, False),
        'SmoothStreaming': (boolean, False),
        'TargetOriginId': (basestring, True),
        'TrustedSigners': (list, False),
        'ViewerProtocolPolicy': (basestring, True),
    }


class S3Origin(AWSProperty):
    props = {
        'OriginAccessIdentity': (basestring, False),
    }


class CustomOrigin(AWSProperty):
    props = {
        'HTTPPort': (network_port, False),
        'HTTPSPort': (network_port, False),
        'OriginKeepaliveTimeout': (integer, False),
        'OriginProtocolPolicy': (basestring, True),
        'OriginSSLProtocols': ([basestring], False),
        'OriginReadTimeout': (integer, False),
    }


class OriginCustomHeader(AWSProperty):
    props = {
        'HeaderName': (basestring, True),
        'HeaderValue': (basestring, True),
    }


class Origin(AWSProperty):
    props = {
        'DomainName': (basestring, True),
        'OriginPath': (basestring, False),
        'Id': (basestring, True),
        'OriginCustomHeaders': ([OriginCustomHeader], False),
        'S3OriginConfig': (S3Origin, False),
        'CustomOriginConfig': (CustomOrigin, False),
    }


class Logging(AWSProperty):
    props = {
        'Bucket': (basestring, True),
        'IncludeCookies': (boolean, False),
        'Prefix': (basestring, False),
    }


class CustomErrorResponse(AWSProperty):
    props = {
        'ErrorCachingMinTTL': (positive_integer, False),
        'ErrorCode': (positive_integer, True),
        'ResponseCode': (positive_integer, False),
        'ResponsePagePath': (basestring, False),
    }


class GeoRestriction(AWSProperty):
    props = {
        'Locations': ([basestring], False),
        'RestrictionType': (basestring, True),
    }


class Restrictions(AWSProperty):
    props = {
        'GeoRestriction': (GeoRestriction, True),
    }


class ViewerCertificate(AWSProperty):
    props = {
        'AcmCertificateArn': (basestring, False),
        'CloudFrontDefaultCertificate': (boolean, False),
        'IamCertificateId': (basestring, False),
        'MinimumProtocolVersion': (basestring, False),
        'SslSupportMethod': (basestring, False),
    }


class DistributionConfig(AWSProperty):
    props = {
        'Aliases': (list, False),
        'CacheBehaviors': ([CacheBehavior], False),
        'Comment': (basestring, False),
        'CustomErrorResponses': ([CustomErrorResponse], False),
        'DefaultCacheBehavior': (DefaultCacheBehavior, True),
        'DefaultRootObject': (basestring, False),
        'Enabled': (boolean, True),
        'HttpVersion': (basestring, False),
        'IPV6Enabled': (boolean, False),
        'Logging': (Logging, False),
        'Origins': (list, True),
        'PriceClass': (basestring, False),
        'Restrictions': (Restrictions, False),
        'ViewerCertificate': (ViewerCertificate, False),
        'WebACLId': (basestring, False),
    }


class Distribution(AWSObject):
    resource_type = "AWS::CloudFront::Distribution"

    props = {
        'DistributionConfig': (DistributionConfig, True),
        'Tags': ((Tags, list), False),
    }


class CloudFrontOriginAccessIdentityConfig(AWSProperty):
    props = {
        'Comment': (basestring, True),
    }


class CloudFrontOriginAccessIdentity(AWSObject):
    resource_type = "AWS::CloudFront::CloudFrontOriginAccessIdentity"

    props = {
        'CloudFrontOriginAccessIdentityConfig': (
            CloudFrontOriginAccessIdentityConfig,
            True,
        ),
    }


class TrustedSigners(AWSProperty):
    props = {
        'AwsAccountNumbers': ([basestring], False),
        'Enabled': (boolean, True),
    }


class StreamingDistributionConfig(AWSProperty):
    props = {
        'Aliases': ([basestring], False),
        'Comment': (basestring, True),
        'Enabled': (boolean, True),
        'Logging': (Logging, False),
        'PriceClass': (basestring, False),
        'S3Origin': (S3Origin, True),
        'TrustedSigners': (TrustedSigners, True),
    }


class StreamingDistribution(AWSObject):
    resource_type = "AWS::CloudFront::StreamingDistribution"

    props = {
        'StreamingDistributionConfig': (StreamingDistributionConfig, True,),
        'Tags': ((Tags, list), False),
    }



from . import AWSObject, AWSProperty
try:
    from awacs.aws import Policy
    policytypes = (dict, Policy)
except ImportError:
    policytypes = dict,


class LifecyclePolicy(AWSProperty):
    props = {
        'LifecyclePolicyText': (basestring, False),
        'RegistryId': (basestring, False),
    }


class Repository(AWSObject):
    resource_type = "AWS::ECR::Repository"

    props = {
        'LifecyclePolicy': (LifecyclePolicy, False),
        'RepositoryName': (basestring, False),
        'RepositoryPolicyText': (policytypes, False),
    }



# Copyright (c) 2014, Guillem Anguera <ganguera@gmail.com>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import boolean, integer


class LoggingProperties(AWSProperty):
    props = {
        'BucketName': (basestring, True),
        'S3KeyPrefix': (basestring, False),
    }


class Cluster(AWSObject):
    resource_type = "AWS::Redshift::Cluster"

    props = {
        'AllowVersionUpgrade': (boolean, False),
        'AutomatedSnapshotRetentionPeriod': (integer, False),
        'AvailabilityZone': (basestring, False),
        'ClusterIdentifier': (basestring, False),
        'ClusterParameterGroupName': (basestring, False),
        'ClusterSecurityGroups': (list, False),
        'ClusterSubnetGroupName': (basestring, False),
        'ClusterType': (basestring, True),
        'ClusterVersion': (basestring, False),
        'DBName': (basestring, True),
        'ElasticIp': (basestring, False),
        'Encrypted': (boolean, False),
        'HsmClientCertificateIdentifier': (basestring, False),
        'HsmConfigurationIdentifier': (basestring, False),
        'IamRoles': ([basestring], False),
        'KmsKeyId': (basestring, False),
        'LoggingProperties': (LoggingProperties, False),
        'MasterUsername': (basestring, True),
        'MasterUserPassword': (basestring, True),
        'NodeType': (basestring, True),
        'NumberOfNodes': (integer, False),  # Conditional
        'OwnerAccount': (basestring, False),
        'Port': (integer, False),
        'PreferredMaintenanceWindow': (basestring, False),
        'PubliclyAccessible': (boolean, False),
        'SnapshotClusterIdentifier': (basestring, False),
        'SnapshotIdentifier': (basestring, False),
        'Tags': (Tags, False),
        'VpcSecurityGroupIds': (list, False),
    }


class AmazonRedshiftParameter(AWSProperty):
    props = {
        'ParameterName': (basestring, True),
        'ParameterValue': (basestring, True),
    }


class ClusterParameterGroup(AWSObject):
    resource_type = "AWS::Redshift::ClusterParameterGroup"

    props = {
        'Description': (basestring, True),
        'ParameterGroupFamily': (basestring, True),
        'Parameters': ([AmazonRedshiftParameter], False),
        'Tags': (Tags, False),
    }


class ClusterSecurityGroup(AWSObject):
    resource_type = "AWS::Redshift::ClusterSecurityGroup"

    props = {
        'Description': (basestring, True),
        'Tags': (Tags, False),
    }


class ClusterSecurityGroupIngress(AWSObject):
    resource_type = "AWS::Redshift::ClusterSecurityGroupIngress"

    props = {
        'ClusterSecurityGroupName': (basestring, True),
        'CIDRIP': (basestring, False),
        'EC2SecurityGroupName': (basestring, False),
        'EC2SecurityGroupOwnerId': (basestring, False),
    }


class ClusterSubnetGroup(AWSObject):
    resource_type = "AWS::Redshift::ClusterSubnetGroup"

    props = {
        'Description': (basestring, True),
        'SubnetIds': (list, True),
        'Tags': (Tags, False),
    }



"""
Batch.

Create the AWS Batch Compute environment and roles.
"""

from troposphere import GetAtt, Output, Parameter, Ref, Tags, Template
from troposphere.batch import (ComputeEnvironment, ComputeEnvironmentOrder,
                               ComputeResources, JobQueue)
from troposphere.ec2 import SecurityGroup
from troposphere.iam import InstanceProfile, Role

t = Template('AWS Batch')
t.add_version()

PrivateSubnetA = t.add_parameter(Parameter(
    'PrivateSubnetA',
    Type='String'
))

PrivateSubnetB = t.add_parameter(Parameter(
    'PrivateSubnetB',
    Type='String'
))

Vpc = t.add_parameter(Parameter(
    'Vpc',
    ConstraintDescription='Must be a valid VPC ID.',
    Type='AWS::EC2::VPC::Id'
))

BatchServiceRole = t.add_resource(Role(
    'BatchServiceRole',
    Path='/',
    Policies=[],
    ManagedPolicyArns=[
        'arn:aws:iam::aws:policy/service-role/AWSBatchServiceRole',
    ],
    AssumeRolePolicyDocument={'Statement': [{
        'Action': ['sts:AssumeRole'],
        'Effect': 'Allow',
        'Principal': {'Service': ['batch.amazonaws.com']}
    }]},
))

BatchInstanceRole = t.add_resource(Role(
    'BatchInstanceRole',
    Path='/',
    Policies=[],
    ManagedPolicyArns=[
        'arn:aws:iam::aws:policy/service-role/AmazonEC2ContainerServiceforEC2Role',  # NOQA
    ],
    AssumeRolePolicyDocument={'Statement': [{
        'Action': ['sts:AssumeRole'],
        'Effect': 'Allow',
        'Principal': {'Service': ['ec2.amazonaws.com']}
    }]},
))

BatchInstanceProfile = t.add_resource(InstanceProfile(
    'BatchInstanceProfile',
    Path='/',
    Roles=[Ref(BatchInstanceRole)],
))

BatchSecurityGroup = t.add_resource(SecurityGroup(
    'BatchSecurityGroup',
    VpcId=Ref(Vpc),
    GroupDescription='Enable access to Batch instances',
    Tags=Tags(Name='batch-sg')
))

BatchComputeEnvironment = t.add_resource(ComputeEnvironment(
    'BatchComputeEnvironment',
    Type='MANAGED',
    ServiceRole=Ref(BatchServiceRole),
    ComputeResources=ComputeResources(
        'BatchComputeResources',
        Type='EC2',
        DesiredvCpus=0,
        MinvCpus=0,
        MaxvCpus=10,
        InstanceTypes=['m4.large'],
        InstanceRole=Ref(BatchInstanceProfile),
        SecurityGroupIds=[GetAtt(BatchSecurityGroup, 'GroupId')],
        Subnets=[
            Ref(PrivateSubnetA),
            Ref(PrivateSubnetB)
        ],
        Tags=dict(
            Name='batch-compute-environment',
            Project='example'
        )
    )
))

ExampleJobQueue = t.add_resource(JobQueue(
    'ExampleJobQueue',
    ComputeEnvironmentOrder=[
        ComputeEnvironmentOrder(
            ComputeEnvironment=Ref(BatchComputeEnvironment),
            Order=1
        ),
    ],
    Priority=1,
    State='ENABLED',
    JobQueueName='example-job-queue'
))

t.add_output([
    Output('BatchComputeEnvironment', Value=Ref(BatchComputeEnvironment)),
    Output('BatchSecurityGroup', Value=Ref(BatchSecurityGroup)),
    Output('ExampleJobQueue', Value=Ref(ExampleJobQueue))
])

print(t.to_json())



# Copyright (c) 2015, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import (integer, boolean, s3_bucket_name, notification_type,
                         notification_event, json_checker, task_type,
                         operating_system, compliance_level)


class NotificationConfig(AWSProperty):
    props = {
        'NotificationArn': (basestring, False),
        'NotificationEvents': (notification_event, False),
        'NotificationType': (notification_type, False),
    }


class LoggingInfo(AWSProperty):
    props = {
        'Region': (basestring, True),
        'S3Bucket': (s3_bucket_name, True),
        'S3Prefix': (basestring, False),
    }


class MaintenanceWindowAutomationParameters(AWSProperty):
    props = {
        'DocumentVersion': (basestring, False),
        'Parameters': (dict, False),
    }


class MaintenanceWindowLambdaParameters(AWSProperty):
    props = {
        'ClientContext': (basestring, False),
        'Payload': (json_checker, False),
        'Qualifier': (basestring, False),
    }


class MaintenanceWindowRunCommandParameters(AWSProperty):
    props = {
        'Comment': (basestring, False),
        'DocumentHash': (basestring, False),
        'DocumentHashType': (basestring, False),
        'NotificationConfig': (NotificationConfig, False),
        'OutputS3BucketName': (s3_bucket_name, False),
        'OutputS3KeyPrefix': (basestring, False),
        'Parameters': (dict, False),
        'ServiceRoleArn': (basestring, False),
        'TimeoutSeconds': (integer, False),
    }


class MaintenanceWindowStepFunctionsParameters(AWSProperty):
    props = {
        'Input': (basestring, False),
        'Name': (basestring, False),
    }


class PatchFilter(AWSProperty):
    props = {
        'Key': (basestring, True),
        'Values': ([basestring], True),
    }


class PatchFilterGroup(AWSProperty):
    props = {
        'PatchFilters': ([PatchFilter], False),
    }


class Rule(AWSProperty):
    props = {
        'ApproveAfterDays': (integer, False),
        'ComplianceLevel': (compliance_level, False),
        'PatchFilterGroup': (PatchFilterGroup, False),
    }


class RuleGroup(AWSProperty):
    props = {
        'PatchRules': ([Rule], False),
    }


class TaskInvocationParameters(AWSProperty):
    props = {
        'MaintenanceWindowAutomationParameters':
        (MaintenanceWindowAutomationParameters, False),
        'MaintenanceWindowLambdaParameters':
        (MaintenanceWindowLambdaParameters, False),
        'MaintenanceWindowRunCommandParameters':
        (MaintenanceWindowRunCommandParameters, False),
        'MaintenanceWindowStepFunctionsParameters':
        (MaintenanceWindowStepFunctionsParameters, False),
    }


class Targets(AWSProperty):
    props = {
        'Key': (basestring, True),
        'Values': ([basestring], True),
    }


class S3OutputLocation(AWSProperty):
    props = {
        'OutputS3BucketName': (basestring, False),
        'OutputS3KeyPrefix': (basestring, False),
    }


class InstanceAssociationOutputLocation(AWSProperty):
    props = {
        'S3Location': (S3OutputLocation, False),
    }


class Association(AWSObject):
    resource_type = "AWS::SSM::Association"

    props = {
        'AssociationName': (basestring, False),
        'DocumentVersion': (basestring, False),
        'InstanceId': (basestring, False),
        'Name': (basestring, True),
        'OutputLocation': (InstanceAssociationOutputLocation, False),
        'Parameters': (dict, False),
        'ScheduleExpression': (basestring, False),
        'Targets': ([Targets], False),
    }


class Document(AWSObject):
    resource_type = "AWS::SSM::Document"

    props = {
        # Need a better implementation of the SSM Document
        'Content': (dict, True),
        'DocumentType': (basestring, False),
        'Tags': (Tags, False),
    }


class MaintenanceWindow(AWSObject):
    resource_type = "AWS::SSM::MaintenanceWindow"

    props = {
        'AllowUnassociatedTargets': (boolean, True),
        'Cutoff': (integer, True),
        'Description': (basestring, False),
        'Duration': (integer, True),
        'Name': (basestring, True),
        'Schedule': (basestring, True),
    }


class MaintenanceWindowTarget(AWSObject):
    resource_type = "AWS::SSM::MaintenanceWindowTarget"

    props = {
        'Description': (basestring, False),
        'Name': (basestring, False),
        'OwnerInformation': (basestring, False),
        'ResourceType': (basestring, True),
        'Targets': ([Targets], True),
        'WindowId': (basestring, True),
    }


class MaintenanceWindowTask(AWSObject):
    resource_type = "AWS::SSM::MaintenanceWindowTask"

    props = {
        'Description': (basestring, False),
        'LoggingInfo': (LoggingInfo, False),
        'MaxConcurrency': (integer, False),
        'MaxErrors': (integer, True),
        'Name': (basestring, False),
        'Priority': (integer, True),
        'ServiceRoleArn': (basestring, True),
        'Targets': ([Targets], True),
        'TaskArn': (basestring, True),
        'TaskInvocationParameters': (TaskInvocationParameters, False),
        'TaskParameters': (dict, False),
        'TaskType': (task_type, True),
        'WindowId': (basestring, False),
    }


class Parameter(AWSObject):
    resource_type = "AWS::SSM::Parameter"

    props = {
        'AllowedPattern': (basestring, False),
        'Description': (basestring, False),
        'Name': (basestring, False),
        'Type': (basestring, True),
        'Value': (basestring, True),
    }


class PatchBaseline(AWSObject):
    resource_type = "AWS::SSM::PatchBaseline"

    props = {
        'ApprovalRules': (RuleGroup, False),
        'ApprovedPatches': ([basestring], False),
        'ApprovedPatchesComplianceLevel': (compliance_level, False),
        'Description': (basestring, False),
        'GlobalFilters': (PatchFilterGroup, False),
        'Name': (basestring, True),
        'OperatingSystem': (operating_system, False),
        'PatchGroups': ([basestring], False),
        'RejectedPatches': ([basestring], False),
    }



from troposphere import Template
from troposphere.codebuild import Artifacts, Environment, Source, Project


template = Template()
template.add_version('2010-09-09')

artifacts = Artifacts(Type='NO_ARTIFACTS')

environment = Environment(
    ComputeType='BUILD_GENERAL1_SMALL',
    Image='aws/codebuild/java:openjdk-8',
    Type='LINUX_CONTAINER',
    EnvironmentVariables=[{'Name': 'APP_NAME', 'Value': 'demo'}],
)

source = Source(
    Location='codebuild-demo-test/0123ab9a371ebf0187b0fe5614fbb72c',
    Type='S3'
)

project = Project(
    "DemoProject",
    Artifacts=artifacts,
    Environment=environment,
    Name='DemoProject',
    ServiceRole='arn:aws:iam::0123456789:role/codebuild-role',
    Source=source,
)
template.add_resource(project)

print(template.to_json())



# Converted from IAM_Policies_SNS_Publish_To_SQS.template located at:
# http://aws.amazon.com/cloudformation/aws-cloudformation-templates/

from troposphere import GetAtt, Output, Ref, Template
from troposphere.sns import Subscription, Topic
from troposphere.sqs import Queue, QueuePolicy


t = Template()

t.add_description("AWS CloudFormation Sample Template: This template "
                  "demonstrates the creation of a DynamoDB table.")

sqsqueue = t.add_resource(Queue("SQSQueue"))

snstopic = t.add_resource(Topic(
    "SNSTopic",
    Subscription=[Subscription(
        Protocol="sqs",
        Endpoint=GetAtt(sqsqueue, "Arn")
    )]
))

t.add_output(Output(
    "QueueArn",
    Value=GetAtt(sqsqueue, "Arn"),
    Description="ARN of SQS Queue",
))

t.add_resource(QueuePolicy(
    "AllowSNS2SQSPolicy",
    Queues=[Ref(sqsqueue)],
    PolicyDocument={
        "Version": "2008-10-17",
        "Id": "PublicationPolicy",
        "Statement": [{
            "Sid": "Allow-SNS-SendMessage",
            "Effect": "Allow",
            "Principal": {
              "AWS": "*"
            },
            "Action": ["sqs:SendMessage"],
            "Resource": GetAtt(sqsqueue, "Arn"),
            "Condition": {
                "ArnEquals": {"aws:SourceArn": Ref(snstopic)}
            }
        }]
    }
))

print(t.to_json())



# Converted from AWS WAF Sample located at:
# https://s3.amazonaws.com/cloudformation-examples/community/common-attacks.json

from troposphere import (
    Template,
    Parameter,
    Join,
    Ref
)
from troposphere.waf import (
    Rule,
    SqlInjectionMatchSet,
    WebACL,
    SizeConstraintSet,
    IPSet,
    XssMatchSet,
    Predicates,
    SqlInjectionMatchTuples,
    FieldToMatch,
    Action,
    Rules,
    SizeConstraint,
    XssMatchTuple
)

t = Template()

t.add_version("2010-09-09")

t.add_description(
    "Creates an AWS WAF configuration that protects against common attacks"
)

WebACLName = t.add_parameter(Parameter(
    "WebACLName",
    Default="CommonAttackProtection",
    Type="String",
    Description="Enter the name you want to use for the WebACL. "
    "This value is also added as a prefix for the names of the rules, "
    "conditions, and CloudWatch metrics created by this template.",
))

SqliMatchSet = t.add_resource(SqlInjectionMatchSet(
    "SqliMatchSet",
    Name=Join("", [Ref(WebACLName), "SqliMatch"]),
    SqlInjectionMatchTuples=[
        SqlInjectionMatchTuples(
            FieldToMatch=FieldToMatch(
                Type="QUERY_STRING"
            ),
            TextTransformation="URL_DECODE"
        ),
        SqlInjectionMatchTuples(
            FieldToMatch=FieldToMatch(
                Type="QUERY_STRING"
            ),
            TextTransformation="HTML_ENTITY_DECODE"
        ),
        SqlInjectionMatchTuples(
            FieldToMatch=FieldToMatch(
                Type="BODY"
            ),
            TextTransformation="URL_DECODE"
        ),
        SqlInjectionMatchTuples(
            FieldToMatch=FieldToMatch(
                Type="BODY"
            ),
            TextTransformation="HTML_ENTITY_DECODE"
        ),
        SqlInjectionMatchTuples(
            FieldToMatch=FieldToMatch(
                Type="URI"
            ),
            TextTransformation="URL_DECODE"
        )
    ]
))

SqliRule = t.add_resource(Rule(
    "SqliRule",
    Predicates=[
        Predicates(
            DataId=Ref(SqliMatchSet),
            Type="SqlInjectionMatch",
            Negated=False
        )
    ],
    Name=Join("", [Ref(WebACLName), "SqliRule"]),
    MetricName=Join("", [Ref(WebACLName), "SqliRule"]),
))

XssMatchSet = t.add_resource(XssMatchSet(
    "XssMatchSet",
    Name=Join("", [Ref(WebACLName), "XssMatch"]),
    XssMatchTuples=[
        XssMatchTuple(
            FieldToMatch=FieldToMatch(
                Type="QUERY_STRING",
            ),
            TextTransformation="URL_DECODE"
        ),
        XssMatchTuple(
            FieldToMatch=FieldToMatch(
                Type="QUERY_STRING",
            ),
            TextTransformation="HTML_ENTITY_DECODE"
        ),
        XssMatchTuple(
            FieldToMatch=FieldToMatch(
                Type="BODY",
            ),
            TextTransformation="URL_DECODE"
        ),
        XssMatchTuple(
            FieldToMatch=FieldToMatch(
                Type="BODY",
            ),
            TextTransformation="HTML_ENTITY_DECODE"
        ),
        XssMatchTuple(
            FieldToMatch=FieldToMatch(
                Type="URI",
            ),
            TextTransformation="URL_DECODE"
        )
    ]
))

XssRule = t.add_resource(Rule(
    "XssRule",
    Name=Join("", [Ref(WebACLName), "XssRule"]),
    Predicates=[
        Predicates(
            DataId=Ref(XssMatchSet),
            Type="XssMatch",
            Negated=False
        )
    ],
    MetricName=Join("", [Ref(WebACLName), "XssRule"]),
))

WAFManualIPBlockSet = t.add_resource(IPSet(
    "WAFManualIPBlockSet",
    Name="Manual IP Block Set",
))

ManualIPBlockRule = t.add_resource(Rule(
    "ManualIPBlockRule",
    Name=Join("", [Ref(WebACLName), "ManualIPBlockRule"]),
    MetricName=Join("", [Ref(WebACLName), "ManualIPBlockRule"]),
    Predicates=[
        Predicates(
            DataId=Ref(WAFManualIPBlockSet),
            Type="IPMatch",
            Negated=False
        )
    ]
))

SizeMatchSet = t.add_resource(SizeConstraintSet(
    "SizeMatchSet",
    Name=Join("", [Ref(WebACLName), "LargeBodyMatch"]),
    SizeConstraints=[
        SizeConstraint(
            ComparisonOperator="GT",
            TextTransformation="NONE",
            FieldToMatch=FieldToMatch(
                Type="BODY"
            ),
            Size="8192"
        )
    ]
))

SizeMatchRule = t.add_resource(Rule(
    "SizeMatchRule",
    Name=Join("", [Ref(WebACLName), "LargeBodyMatchRule"]),
    MetricName=Join("", [Ref(WebACLName), "DetectLargeBody"]),
    Predicates=[
        Predicates(
            DataId=Ref(SizeMatchSet),
            Type="SizeConstraint",
            Negated=False
        )
    ]
))

MyWebACL = t.add_resource(WebACL(
    "MyWebACL",
    Name=Ref(WebACLName),
    DefaultAction=Action(
        Type="ALLOW"
    ),
    Rules=[
        Rules(
            Action=Action(
                Type="BLOCK"
            ),
            Priority=1,
            RuleId=Ref(ManualIPBlockRule)
        ),
        Rules(
            Action=Action(
                Type="COUNT"
            ),
            Priority=2,
            RuleId=Ref(SizeMatchRule)
        ),
        Rules(
            Action=Action(
                Type="BLOCK"
            ),
            Priority=3,
            RuleId=Ref(SqliRule)
        ),
        Rules(
            Action=Action(
                Type="BLOCK"
            ),
            Priority=4,
            RuleId=Ref(XssRule)
        )
    ],
    MetricName=Ref(WebACLName),
))

print(t.to_json())



# Copyright (c) 2012-2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, Tags
from .validators import boolean
try:
    from awacs.aws import Policy
    policytypes = (dict, Policy)
except ImportError:
    policytypes = dict,


class Alias(AWSObject):
    resource_type = "AWS::KMS::Alias"

    props = {
        'AliasName': (basestring, True),
        'TargetKeyId': (basestring, True)
    }


class Key(AWSObject):
    resource_type = "AWS::KMS::Key"

    props = {
        'Description': (basestring, False),
        'Enabled': (boolean, False),
        'EnableKeyRotation': (boolean, False),
        'KeyPolicy': (policytypes, True),
        'Tags': ((Tags, list), False)
    }



# Copyright (c) 2012-2013, Mark Peek <mark@peek.org>
# Copyright (c) 2014, Andy Botting <andy.botting@theguardian.com>
# All rights reserved.
#
# See LICENSE file for full license.

from troposphere import AWSObject
from troposphere.validators import integer


# Due to the strange nature of the OpenStack compatability layer, some values
# that should be integers fail to validate and need to be represented as
# strings. For this reason, we duplicate the AWS::AutoScaling::AutoScalingGroup
# and change these types.
class AWSAutoScalingGroup(AWSObject):
    resource_type = "AWS::AutoScaling::AutoScalingGroup"

    props = {
        'AvailabilityZones': (list, True),
        'Cooldown': (integer, False),
        'DesiredCapacity': (basestring, False),
        'HealthCheckGracePeriod': (integer, False),
        'HealthCheckType': (basestring, False),
        'LaunchConfigurationName': (basestring, True),
        'LoadBalancerNames': (list, False),
        'MaxSize': (basestring, True),
        'MinSize': (basestring, True),
        'Tags': (list, False),
        'VPCZoneIdentifier': (list, False),
    }



# Copyright (c) 2015, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty
from .validators import boolean, integer


class ActionTypeId(AWSProperty):
    props = {
        'Category': (basestring, True),
        'Owner': (basestring, True),
        'Provider': (basestring, True),
        'Version': (basestring, True)
    }


class ArtifactDetails(AWSProperty):
    props = {
        'MaximumCount': (integer, True),
        'MinimumCount': (integer, True)
    }


class Blockers(AWSProperty):
    props = {
        'Name': (basestring, True),
        'Type': (basestring, True)
    }


class ConfigurationProperties(AWSProperty):
    props = {
        'Description': (basestring, False),
        'Key': (boolean, True),
        'Name': (basestring, True),
        'Queryable': (boolean, False),
        'Required': (boolean, True),
        'Secret': (boolean, True),
        'Type': (basestring, False)
    }


class EncryptionKey(AWSProperty):
    props = {
        'Id': (basestring, True),
        'Type': (basestring, True)
    }


class DisableInboundStageTransitions(AWSProperty):
    props = {
        'Reason': (basestring, True),
        'StageName': (basestring, True)
    }


class InputArtifacts(AWSProperty):
    props = {
        'Name': (basestring, True)
    }


class OutputArtifacts(AWSProperty):
    props = {
        'Name': (basestring, True)
    }


class Settings(AWSProperty):
    props = {
        'EntityUrlTemplate': (basestring, False),
        'ExecutionUrlTemplate': (basestring, False),
        'RevisionUrlTemplate': (basestring, False),
        'ThirdPartyConfigurationUrl': (basestring, False)
    }


class ArtifactStore(AWSProperty):
    props = {
        'EncryptionKey': (EncryptionKey, False),
        'Location': (basestring, True),
        'Type': (basestring, True)
    }


class Actions(AWSProperty):
    props = {
        'ActionTypeId': (ActionTypeId, True),
        'Configuration': (dict, False),
        'InputArtifacts': ([InputArtifacts], False),
        'Name': (basestring, True),
        'OutputArtifacts': ([OutputArtifacts], False),
        'RoleArn': (basestring, False),
        'RunOrder': (integer, False)
    }


class Stages(AWSProperty):
    props = {
        'Actions': ([Actions], True),
        'Blockers': ([Blockers], False),
        'Name': (basestring, True)
    }


class CustomActionType(AWSObject):
    resource_type = "AWS::CodePipeline::CustomActionType"

    props = {
        'Category': (basestring, True),
        'ConfigurationProperties': ([ConfigurationProperties], False),
        'InputArtifactDetails': (ArtifactDetails, True),
        'OutputArtifactDetails': (ArtifactDetails, True),
        'Provider': (basestring, True),
        'Settings': (Settings, False),
        'Version': (basestring, False)
    }


class Pipeline(AWSObject):
    resource_type = "AWS::CodePipeline::Pipeline"

    props = {
        'ArtifactStore': (ArtifactStore, True),
        'DisableInboundStageTransitions':
            ([DisableInboundStageTransitions], False),
        'Name': (basestring, False),
        'RestartExecutionOnUpdate': (boolean, False),
        'RoleArn': (basestring, True),
        'Stages': ([Stages], True)
    }



# Copyright (c) 2017, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import boolean, integer, network_port, positive_integer


CDC = "cdc"
FULL_LOAD = "full-load"
FULL_LOAD_AND_CDC = "full-load-and-cdc"


class Certificate(AWSObject):
    resource_type = "AWS::DMS::Certificate"

    props = {
        'CertificateIdentifier': (basestring, False),
        'CertificatePem': (basestring, False),
        'CertificateWallet': (basestring, False),
    }


class DynamoDBSettings(AWSProperty):
    props = {
        'ServiceAccessRoleArn': (basestring, True),
    }


class MongoDbSettings(AWSProperty):
    props = {
        'AuthMechanism': (basestring, False),
        'AuthSource': (basestring, False),
        'DatabaseName': (basestring, False),
        'DocsToInvestigate': (basestring, False),
        'ExtractDocId': (basestring, False),
        'KmsKeyId': (basestring, False),
        'NestingLevel': (basestring, False),
        'Password': (basestring, False),
        'Port': (network_port, False),
        'ServerName': (basestring, False),
        'Username': (basestring, False),
    }


class S3Settings(AWSProperty):
    props = {
        'BucketFolder': (basestring, False),
        'BucketName': (basestring, False),
        'CompressionType': (basestring, False),
        'CsvDelimiter': (basestring, False),
        'CsvRowDelimiter': (basestring, False),
        'ExternalTableDefinition': (basestring, False),
        'ServiceAccessRoleArn': (basestring, False),
    }


class Endpoint(AWSObject):
    resource_type = "AWS::DMS::Endpoint"

    props = {
        'CertificateArn': (basestring, False),
        'DatabaseName': (basestring, False),
        'DynamoDbSettings': (DynamoDBSettings, False),
        'EndpointType': (basestring, True),
        'EngineName': (basestring, True),
        'ExtraConnectionAttributes': (basestring, False),
        'KmsKeyId': (basestring, False),
        'MongoDbSettings': (MongoDbSettings, False),
        'Password': (basestring, False),
        'Port': (network_port, False),
        'S3Settings': (S3Settings, False),
        'ServerName': (basestring, False),
        'SslMode': (basestring, False),
        'Tags': (Tags, False),
        'Username': (basestring, True),
    }


class EventSubscription(AWSObject):
    resource_type = "AWS::DMS::EventSubscription"

    props = {
        'Enabled': (boolean, False),
        'EventCategories': ([basestring], False),
        'SnsTopicArn': (basestring, True),
        'SourceIds': ([basestring], False),
        'SourceType': (basestring, False),
        'SubscriptionName': ([basestring], False),
        'Tags': (Tags, False),
    }


class ReplicationInstance(AWSObject):
    resource_type = "AWS::DMS::ReplicationInstance"

    props = {
        'AllocatedStorage': (integer, False),
        'AutoMinorVersionUpgrade': (boolean, False),
        'AvailabilityZone': (basestring, False),
        'EngineVersion': (basestring, False),
        'KmsKeyId': (basestring, False),
        'MultiAZ': (boolean, False),
        'PreferredMaintenanceWindow': (basestring, False),
        'PubliclyAccessible': (boolean, False),
        'ReplicationInstanceClass': (basestring, True),
        'ReplicationInstanceIdentifier': (basestring, False),
        'ReplicationSubnetGroupIdentifier': (basestring, False),
        'Tags': (Tags, False),
        'VpcSecurityGroupIds': ([basestring], False),
    }


class ReplicationSubnetGroup(AWSObject):
    resource_type = "AWS::DMS::ReplicationSubnetGroup"

    props = {
        'ReplicationSubnetGroupIdentifier': (basestring, False),
        'ReplicationSubnetGroupDescription': (basestring, True),
        'SubnetIds': ([basestring], True),
        'Tags': (Tags, False),
    }


class ReplicationTask(AWSObject):
    resource_type = "AWS::DMS::ReplicationTask"

    props = {
        'CdcStartTime': (positive_integer, False),
        'MigrationType': (basestring, True),
        'ReplicationInstanceArn': (basestring, True),
        'ReplicationTaskIdentifier': (basestring, False),
        'ReplicationTaskSettings': (basestring, False),
        'SourceEndpointArn': (basestring, True),
        'TableMappings': (basestring, True),
        'Tags': (Tags, False),
        'TargetEndpointArn': (basestring, True),
    }



# Copyright (c) 2012-2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject


class Activity(AWSObject):
    resource_type = "AWS::StepFunctions::Activity"
    props = {
        'Name': (basestring, True),
    }


class StateMachine(AWSObject):
    resource_type = "AWS::StepFunctions::StateMachine"
    props = {
        'StateMachineName': (basestring, False),
        'DefinitionString': (basestring, True),
        'RoleArn': (basestring, True),

    }



from troposphere import Parameter, Ref, Template
from troposphere.ecs import (
    Cluster, Service, TaskDefinition,
    ContainerDefinition, NetworkConfiguration,
    AwsvpcConfiguration, PortMapping
)

t = Template()
t.add_version('2010-09-09')
t.add_parameter(Parameter(
    'Subnet',
    Type='AWS::EC2::Subnet::Id',
    Description='A VPC subnet ID for the container.',
))

cluster = t.add_resource(Cluster(
    'Cluster'
))

task_definition = t.add_resource(TaskDefinition(
    'TaskDefinition',
    RequiresCompatibilities=['FARGATE'],
    Cpu='256',
    Memory='512',
    NetworkMode='awsvpc',
    ContainerDefinitions=[
        ContainerDefinition(
            Name='nginx',
            Image='nginx',
            Essential=True,
            PortMappings=[PortMapping(ContainerPort=80)]
        )
    ]
))

service = t.add_resource(Service(
    'NginxService',
    Cluster=Ref(cluster),
    DesiredCount=1,
    TaskDefinition=Ref(task_definition),
    LaunchType='FARGATE',
    NetworkConfiguration=NetworkConfiguration(
        AwsvpcConfiguration=AwsvpcConfiguration(
            Subnets=[Ref('Subnet')]
        )
    )
))

print(t.to_json())



# Copyright (c) 2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty
from .validators import boolean, exactly_one, json_checker, positive_integer


class MetricDimension(AWSProperty):
    props = {
        'Name': (basestring, True),
        'Value': (basestring, True),
    }


class Alarm(AWSObject):
    resource_type = "AWS::CloudWatch::Alarm"

    props = {
        'ActionsEnabled': (boolean, False),
        'AlarmActions': ([basestring], False),
        'AlarmDescription': (basestring, False),
        'AlarmName': (basestring, False),
        'ComparisonOperator': (basestring, True),
        'Dimensions': ([MetricDimension], False),
        'EvaluateLowSampleCountPercentile': (basestring, False),
        'EvaluationPeriods': (positive_integer, True),
        'ExtendedStatistic': (basestring, False),
        'InsufficientDataActions': ([basestring], False),
        'MetricName': (basestring, True),
        'Namespace': (basestring, True),
        'OKActions': ([basestring], False),
        'Period': (positive_integer, True),
        'Statistic': (basestring, False),
        'Threshold': (basestring, True),
        'TreatMissingData': (basestring, False),
        'Unit': (basestring, False),
    }

    def validate(self):
        conds = [
            'ExtendedStatistic',
            'Statistic',
        ]
        exactly_one(self.__class__.__name__, self.properties, conds)


class Dashboard(AWSObject):
    resource_type = "AWS::CloudWatch::Dashboard"

    props = {
        'DashboardBody': ((basestring, dict), True),
        'DashboardName': (basestring, False),
    }

    def validate(self):
        name = 'DashboardBody'
        if name in self.properties:
            dashboard_body = self.properties.get(name)
            self.properties[name] = json_checker(name, dashboard_body)



# Copyright (c) 2012-2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import integer, positive_integer, network_port, boolean


class AliasTarget(AWSProperty):
    props = {
        'HostedZoneId': (basestring, True),
        'DNSName': (basestring, True),
        'EvaluateTargetHealth': (boolean, False)
    }

    def __init__(self,
                 hostedzoneid=None,
                 dnsname=None,
                 evaluatetargethealth=None,
                 **kwargs):
        # provided for backward compatibility
        if hostedzoneid is not None:
            kwargs['HostedZoneId'] = hostedzoneid
        if dnsname is not None:
            kwargs['DNSName'] = dnsname
        if evaluatetargethealth is not None:
            kwargs['EvaluateTargetHealth'] = evaluatetargethealth
        super(AliasTarget, self).__init__(**kwargs)


class GeoLocation(AWSProperty):
    props = {
        'ContinentCode': (basestring, False),
        'CountryCode': (basestring, False),
        'SubdivisionCode': (basestring, False),
    }


class BaseRecordSet(object):
    props = {
        'AliasTarget': (AliasTarget, False),
        'Comment': (basestring, False),
        'Failover': (basestring, False),
        'GeoLocation': (GeoLocation, False),
        'HealthCheckId': (basestring, False),
        'HostedZoneId': (basestring, False),
        'HostedZoneName': (basestring, False),
        'Name': (basestring, True),
        'Region': (basestring, False),
        'ResourceRecords': (list, False),
        'SetIdentifier': (basestring, False),
        'TTL': (integer, False),
        'Type': (basestring, True),
        'Weight': (integer, False),
    }


class RecordSetType(AWSObject, BaseRecordSet):
    # This is a top-level resource
    resource_type = "AWS::Route53::RecordSet"


class RecordSet(AWSProperty, BaseRecordSet):
    # This is for use in a list with RecordSetGroup (below)
    pass


class RecordSetGroup(AWSObject):
    resource_type = "AWS::Route53::RecordSetGroup"

    props = {
        'HostedZoneId': (basestring, False),
        'HostedZoneName': (basestring, False),
        'RecordSets': (list, False),
        'Comment': (basestring, False),
    }


class AlarmIdentifier(AWSProperty):
    props = {
        'Name': (basestring, True),
        'Region': (basestring, True),
    }


class HealthCheckConfiguration(AWSProperty):
    props = {
        'AlarmIdentifier': (AlarmIdentifier, False),
        'ChildHealthChecks': ([basestring], False),
        'EnableSNI': (boolean, False),
        'FailureThreshold': (positive_integer, False),
        'FullyQualifiedDomainName': (basestring, False),
        'HealthThreshold': (positive_integer, False),
        'InsufficientDataHealthStatus': (basestring, False),
        'Inverted': (boolean, False),
        'IPAddress': (basestring, False),
        'MeasureLatency': (boolean, False),
        'Port': (network_port, False),
        'Regions': ([basestring], False),
        'RequestInterval': (positive_integer, False),
        'ResourcePath': (basestring, False),
        'SearchString': (basestring, False),
        'Type': (basestring, True),
    }


class HealthCheck(AWSObject):
    resource_type = "AWS::Route53::HealthCheck"

    props = {
        'HealthCheckConfig': (HealthCheckConfiguration, True),
        'HealthCheckTags': (Tags, False),
    }


class HostedZoneConfiguration(AWSProperty):
    props = {
        'Comment': (basestring, False),
    }


class HostedZoneVPCs(AWSProperty):
    props = {
        'VPCId': (basestring, True),
        'VPCRegion': (basestring, True),
    }


class QueryLoggingConfig(AWSProperty):
    props = {
        'CloudWatchLogsLogGroupArn': (basestring, True),
    }


class HostedZone(AWSObject):
    resource_type = "AWS::Route53::HostedZone"

    props = {
        'HostedZoneConfig': (HostedZoneConfiguration, False),
        'HostedZoneTags': (Tags, False),
        'Name': (basestring, True),
        'QueryLoggingConfig': (QueryLoggingConfig, False),
        'VPCs': ([HostedZoneVPCs], False),
    }
